{
  SendMail is a simple program to replace the DECnet MAIL network
  object to allow routing of non-VMS mail messages into and out of
  the system.  SendMail works as follows:

    Incoming mail is read following the MAIL object protocol as
    closely as possible.  If this changes in a future release, this
    code MUST be altered as well.

    The conversation between the master MAIL task and this slave is
    as follows (Ref: Osudar, John, "The NJE Mail Bridge...",
		     Proceedings of DECUS USA, Spring 1983, pg. 183).

	master mail			slaved mail
	-----------			-----------
	send SENDERNAME
	send ADDRESSEE 1	      - send success (+1) if valid,
					failure (even) otherwise, followed
					by message text lines.  The end
					of all message text lines is
					delineated by a NULL byte.
	send ADDRESSEE 2	      - as above
	send ADDRESSEE n	      - as above
	send NULL byte
	send "To:" line
	send "Subj:" line
	send text lines
	send NULL byte		      - for each valid addressee, send success
					(+1) if delivered, failure (even) if
					not delivered.  For each failure, send
					error text.  End of error messages is
					flagged by a NULL byte.

    SendMail scans the addressee lines that it receives and looks
    for recognizable foreign addresses.  Currently, any quoted
    address is assumed to be foreign.  Non-foreign addresses are
    passed through to a local task running the normal VMS mailer in
    slaved mode.

    All information is passed transparently between the master task
    and the non-foreign VMS task slaved to SendMail.
}
[inherit ('SYS$LIBRARY:STARLET')]
program SendMail (Master, Slave, UUCP_message, Output);

const
  Maxline = 256;
  Maxnames = 100;
  Master_taskname = 'SYS$NET';
  VMS_Mailer_taskname = 'SYS$NODE::"TASK=VMSMAILER"';

type
  String = varying [Maxline] of Char;
  Address_kind = (Non_foreign, UUCP);
  Alist_ptr = ^Alist;
  Alist = record
	    Next : Alist_ptr;
	    Kind : Address_kind;
	    Valid, Delivered : Boolean;
	    Name : String;
	  end;

var
  Master, Slave, UUCP_message, UUCP_comfile, UUCP_requests : Text;
  Sendername, Address, Message_line, UUCP_host_name : String;
  Text_file_name, Com_file_name : String;
  Activated_mailers : set of Address_kind := [];
  Address_list, Walker : Alist_ptr := Nil;
  Success : [readonly] String := ''(1, 0, 0, 0);
  Null_byte : [readonly] String := ''(0);

{-------------------------   External routines   -----------------------------}

  function LIB$SYS_TRNLOG (Logical_name : Varying [Max1] of Char;
			   var Dst_len : Integer := %Immed 0;
			   var Dest_string : Varying [Max2] of Char;
			   var Table : Integer := %Immed 0;
			   var Acc_mode : Integer := %Immed 0;
			   dsb_mask : Integer := 0
			  ) : Integer; Extern;

{------------------------------------------------------------------------------
  "Strings_are_equal" compares two varying strings, even if they are different
  lengths.  If the lengths are not identical, then they are immediately
  considered non equal.
}
  function Strings_are_equal (Str1 : varying [Len1] of Char;
			      Str2 : varying [Len2] of Char) : Boolean;

  begin (* Strings_are_equal *)
    Strings_are_equal := False;
    if Length (Str1) = Length (Str2) then
      if Str1 = Str2 then Strings_are_equal := True;
  end; (* Strings_are_equal *)

{------------------------------------------------------------------------------
  "Get_filename" finds the NAM block in under a Pascal file variable and
  reads the complete filename out of it.
}
  procedure Get_filename (var Filev : [unsafe] Text;  var Name : String);

  type
    FAB_ptr = ^FAB$TYPE;
    Packedstr = packed array [1..Maxline] of Char;

  var
    File_FAB : FAB_ptr;
    NAM_ptr : [unsafe] ^NAM$TYPE;
    Str_ptr : [unsafe] ^Packedstr;

    function PAS$FAB (var F : [unsafe] Text) : FAB_ptr; Extern;

  begin (* Get_filename *)
    File_FAB := PAS$FAB (Filev);
    NAM_ptr := File_FAB^.FAB$L_NAM;
    Name.Length := NAM_ptr^.NAM$B_RSL;
    Str_ptr := NAM_ptr^.NAM$L_RSA;
    Name.Body := Str_ptr^;
  end; (* Get_filename *)

{------------------------------------------------------------------------------
  "Send" and "Receive" are procedures to read or write messages between
  network tasks.
}
  procedure Send (var Task_variable : Text;
		  Message : String);

  begin (* Send *)
    rewrite (Task_variable);
    writeln (Task_variable, Message);
  end; (* Send *)

  procedure Receive (var Task_variable : Text;
		     var Message : String);

  begin (* Receive *)
    reset (Task_variable);
    readln (Task_variable, Message);
  end; (* Receive *)

{------------------------------------------------------------------------------
  "Scan_address" examines a string and tries to decide what kind of address it
  is.  Currently it only returns "UUCP" or "Non_foreign".  This code should be
  expanded to look for other types if you want to talk to other kinds of
  mailers.

  "Scan_address" is also responsible for tidying up the string if it needs it.
  This code should lower-caseify the UUCP string unless you have the patched
  version of MAIL that makes it not upcase quoted usernames.
}
  function Scan_address (var Address : string) : Address_kind;

  var
    I : Integer;

  begin (* Scan_address *)
    if Index (Address, '!') <> 0 then
      Scan_address := UUCP
    else
      Scan_address := Non_foreign;
  end; (* Scan_address *)

{------------------------------------------------------------------------------
  "Validate" is called to validate an address.  Validation depends on the
  type of address we have.  Non-foreign addresses start a slaved VMS MAIL
  task and pass the address.  Currently, all others are immediately assumed
  valid.  Please note that validate is responsible for notifying the master
  task immediately of the validity of the address.  This includes returning
  the status lines from the VMS MAIL slave if a non-foreign address is
  invalid!

  We are passed a pointer to the address list entry.
}
  procedure Validate (Address_ptr : Alist_ptr);

  begin (* Validate *)
    case Address_ptr^.Kind of
    Non_foreign : begin
		    if not (Non_foreign in Activated_mailers) then begin
		      Open (File_variable := Slave,
			    File_name := VMS_Mailer_taskname,
			    History := OLD,
			    Record_length := 512);
		      Send (Slave, Sendername);
		      Activated_mailers := Activated_mailers + [Non_foreign];
		    end;
		    Send (Slave, Address_ptr^.Name);
		    Receive (Slave, Message_line);
		    Send (Master, Message_line);
		    if Strings_are_equal (Message_line, Success) then
		      Address_ptr^.Valid := True
		    else begin
		      Address_ptr^.Valid := False;
		      repeat
			Receive (Slave, Message_line);
			Send (Master, Message_line);
		      until Strings_are_equal (Message_line, Null_byte);
		    end;
		  end;
    UUCP :	  begin
		    Address_ptr^.Valid := True;
		    Send (Master, Success);
		  end;
    end;
  end; (* Validate *)


{------------------------------------------------------------------------------
  "Got_address" reads the next address from the master task.  It returns
  False if it got a null byte, meaning there are no more addresses.  The
  address is added to the linked list of addresses.  The "Validate"
  routine is then called to check the address and return a status to
  the master task.
}
  function Got_address : Boolean;

  begin (* Got_address *)
    Receive (Master, Address);
    if Strings_are_equal (Address, Null_byte) then
      Got_address := False 
    else begin
      if Address_list = Nil then begin
	New (Address_list);
	Walker := Address_list; end
      else begin
	New (Walker^.Next);
	Walker := Walker^.Next;
      end;
      Walker^.Next := Nil;
      Walker^.Kind := Scan_address (Address);
      Walker^.Name := Address;
      Validate (Walker);
Writeln ('---------------------------------------');
Writeln ('Address: ', Walker^.Name);
Writeln ('Valid: ', Walker^.Valid);
Writeln ('Kind:  ', Walker^.Kind);
      Got_address := True;
    end;
  end; (* Got_address *)

{------------------------------------------------------------------------------
  "Activate_mailers" scans the address list and does any initialization
  that needs to be done.  Some mailers may have been started already by the
  "Validate" procedure.  If so, then the appropriate flag in
  "Activated_mailers"  will already have been set.   These mailers are handled
  before we walk down the address list.  Other mailers are found by walking
  down the list.  These are activated as they are found.
}
  procedure Activate_mailers;

  var
    Delimeter : Integer;
    Todays_date, Current_time : packed array [1..11] of Char;

  begin (* Activate_mailers *)
    if Non_foreign in Activated_mailers then
      Send (Slave, Null_byte);
    Walker := Address_list;
    while Walker <> Nil do begin
      case Walker^.Kind of

      Non_foreign : ;
      UUCP	  : begin
		      if not (UUCP in Activated_mailers) then begin
			Activated_mailers := Activated_mailers + [UUCP];
			Open (File_variable := UUCP_message,
			      File_name := 'SYS$SCRATCH:UUCPmail.txt',
			      History := NEW,
			      Record_length := 512);
			Rewrite (UUCP_message);
			Get_filename (UUCP_message, Text_file_name);
			Date (Todays_date);
			Time (Current_time);
			LIB$SYS_TRNLOG ('UUCP_HOST_NAME',,
					UUCP_host_name);
			Writeln (UUCP_message, 'From ', Sendername, ' ',
				 Todays_date, ' ', Current_time:8, ' ',
				 ' remote from ', UUCP_host_name);

			Open (File_variable := UUCP_comfile,
			      File_name := 'SYS$SCRATCH:UUCPmail.com',
			      History := NEW,
			      Record_length := 512);
			Rewrite (UUCP_comfile);
			Get_filename (UUCP_comfile, Com_file_name);
		      end;
		      Writeln (UUCP_comfile, '$ define/user SYS$INPUT ',
			       Text_file_name);
		      Delimeter := Index (Walker^.Name, '!');
		      Writeln (UUCP_comfile, '$ uux "-" ',
			       Substr (Walker^.Name, 1, Delimeter),
			       'rmail" "(',
			       Substr (Walker^.Name, Delimeter + 1,
				       Length (Walker^.Name) - Delimeter - 1),
			       ')"');
		    end;
      end;
      Walker := Walker^.Next;
    end;
  end; (* Activate_mailers *)

{------------------------------------------------------------------------------
  "Send_message" processes the incoming text and sends it to each mailer
  that has been activated.
}
  procedure Send_message;

  begin (* Send_message *)

{ First the "To:" line }

    Receive (Master, Message_line);
    if Non_foreign in Activated_mailers then
      Send (Slave, Message_line);
    if UUCP in Activated_mailers then
      Writeln (UUCP_message, 'To: ', Message_line);

{ Now the "Subj:" line }

    Receive (Master, Message_line);
    if Non_foreign in Activated_mailers then
      Send (Slave, Message_line);
    if UUCP in Activated_mailers then begin
      Writeln (UUCP_message, 'Subject: ', Message_line);
      Writeln (UUCP_message);
    end;

{ Now the text body }

    Receive (Master, Message_line);
    while not Strings_are_equal (Message_line, Null_byte) do begin
      if Non_foreign in Activated_mailers then
	Send (Slave, Message_line);
      if UUCP in Activated_mailers then
	Writeln (UUCP_message, Message_line);
      Receive (Master, Message_line);
    end;
    if Non_foreign in Activated_mailers then
      Send (Slave, Null_byte);
  end; (* Send_message *)

{------------------------------------------------------------------------------
  "Deliver_message" walks the address list and does whatever needs to be
  done for every valid address in order for the mailer to complete mail
  delivery.  We have to send a delivery status back to our master.

  Non_foreign : Read delivery status directly from slave.
  UUCP        : Start a UUX job.  For now, delivery is always successful.
}
  procedure Deliver_message;

  begin (* Deliver_message *)
    Walker := Address_list;
    while Walker <> Nil do begin
      if Walker^.Valid then
	case Walker^.Kind of

	Non_foreign : begin
			Receive (Slave, Message_line);
			Send (Master, Message_line);
			if not Strings_are_equal (Message_line, Success) then
			  repeat
			    Receive (Slave, Message_line);
			    Send (Master, Message_line);
			  until Strings_are_equal (Message_line, Null_byte);
		      end;
	UUCP	    : Send (Master, Success);
	end;
      Walker := Walker^.Next;
    end;
    if Non_foreign in Activated_mailers then
      Close (Slave);
    if UUCP in Activated_mailers then begin
      Close (UUCP_message);
      Writeln (UUCP_comfile, '$ delete ', Text_file_name);
      Close (UUCP_comfile);
      Open (File_variable := UUCP_requests,
	    File_name := 'UUCP_REQUESTS',
	    History := OLD);
      Rewrite (UUCP_requests);
      Writeln (UUCP_requests, '@UUCP_ROOT:[LIB]DOCOMMAND ', Com_file_name,
	       ' "@SYS$INPUT" DELETE');
      Close (UUCP_requests);
    end;
  end; (* Deliver_message *)


begin (* SendMail *)
  Open (File_variable := Master,
	File_name := Master_taskname,
	History := OLD,
	Record_length := 512);

  Receive (Master, Sendername);

  while Got_address do;
  Activate_mailers;
  Send_message;
  Deliver_message;
  Close (Master);
end. (* SendMail *)
