/*
 *    XDCL	- An extended DCL.
 *
 *    Written by Kevin Carosso @ Hughes S&CG PWB/ENG
 *    November 10, 1982.
 *
 *    Extended DCL is a simple extension to DCL to allow
 *    DCL commands to be executed by a subprocess and
 *    their results (i.e. SYS$OUTPUT for the subprocess)
 *    substituted for any input file to any other
 *    DCL command. This allows the utility of UNIX pipes
 *    to be implemented in a comfortable way under VMS.
 *
 *    The DCL command that is to be substituted for the
 *    input file specification is simply enclosed in an
 *    open bracket-close bracket pair:
 *
 *		{Any DCL command}
 *
 *    For example:
 *
 *		$ XDCL SEARCH {DIRECTORY/OWNER} "[020,001]"
 *
 *    Would execute "DIRECTORY/OWNER" on a subprocess
 *    and direct it's output into the SEARCH command.
 *
 */

#include descrip
#include ssdef
#include iodef

#define FALSE  0
#define TRUE  -1
#define WAIT   TRUE
#define NOWAIT FALSE

extern killproc();
int reason;
struct d {
	     int flink;
	     int exit_handler;
	     int arg_header;
	     int *rsnptr;
	 } desblk = { 0, killproc, 1, &reason };

struct procinfo {
		    int id;
		    char *output_ppf;
		    struct procinfo *next;
		};

struct procinfo *proclist = 0;
char command[256];
int comindex = 0;
$DESCRIPTOR (commdescr, command);

main()
{
    char this_command[256], *mailbox, *crembx();
    int index = 0, quoted = FALSE, status, i;

    $DESCRIPTOR (clival, "COMMAND");

    if (!((status = cli$get_value (&clival, &commdescr)) & SS$_NORMAL))
	exit (status);

    for (i = 255; (command[i] == ' ') || (command[i] == '\0') ; i--);
    commdescr.dsc$w_length = i;

    if (!((status = sys$dclexh (&desblk)) & SS$_NORMAL))
	exit (status);

    while (comindex <= commdescr.dsc$w_length)
	switch (command[comindex])
	{
	    case '{' : if (quoted)
			   this_command[index++] = command[comindex++];
		       else
		       {
			   comindex++;
			   mailbox = crembx ();
			   do_command (mailbox);
			   i = 0;
			   while (this_command[index++] = mailbox[i++]);
			   --index;
		       }
		       break;

	    case '}' : if (quoted)
			   this_command[index++] = command[comindex++];
		       else
			   error ("Syntax error: matching \"{\" not found.");
		       break;

	    case '"' : quoted = !quoted;

	    default  : this_command[index++] = command[comindex++];
	}

    this_command[index] = '\0';
    spawn (this_command, "SYS$OUTPUT", WAIT);
}

do_command(sysoutput)
    char *sysoutput;
{
    extern char command[256];
    extern int comindex;
    extern struct dsc$descriptor_s commdescr;

    char this_command[256], *mailbox, *crembx();
    int index = 0, quoted = FALSE, i;

    do
	if (comindex > commdescr.dsc$w_length)
	   error ("Syntax error: matching \"}\" not found.");
	else
	    switch (command[comindex])
	    {
	        case '{' : if (quoted)
		 	       this_command[index++] = command[comindex++];
		           else
		           {
			       comindex++;
			       mailbox = crembx ();
			       do_command (mailbox);
			       i = 0;
			       while (this_command[index++] = mailbox[i++]);
			       --index;
		           }
		           break;

	        case '"' : quoted = !quoted;

	        default  : this_command[index++] = command[comindex++];
	    }
    while (command[comindex] != '}');

    comindex++;
    this_command[index] = '\0';
    spawn (this_command, sysoutput, NOWAIT);
}

char *crembx ()
{
    static int box_number = 0;
    char *lognam;
    $DESCRIPTOR (logdescr, lognam);
    short chan;
    int status;

    logdescr.dsc$a_pointer = lognam = (char *) calloc (11, sizeof (char));
    sprintf (lognam, "XDCL_%d", box_number++);
    logdescr.dsc$w_length = strlen (lognam);

    if (!((status = sys$crembx (0, &chan, 0, 0, 0xFF0F, 0, &logdescr)) &
	SS$_NORMAL))
	exit (status);

/*
 * In order to work properly with some utilities, a file name as well
 * as a device name is required.  We therefore add ":A." to the mailbox
 * logical name.
 */

    return (strcat (lognam, ":A."));
}

error (message)
    char *message;
{
    printf ("%s\n", message);
    exit (0x10000008);
}

terminator (proc)
    struct procinfo *proc;
{
    int iosb[2], status;
    short chan;
    $DESCRIPTOR (outdescr, proc->output_ppf);

    outdescr.dsc$w_length = strlen (proc->output_ppf);
    if (!((status = sys$assign (&outdescr, &chan, 0, 0)) & SS$_NORMAL))
	exit (status);
    if (!((status = sys$qiow (0, chan, IO$_WRITEOF, &iosb, 0, 0,
			      0, 0, 0, 0, 0, 0))
	  & SS$_NORMAL))
	exit (status);
    if (!(iosb[0] & SS$_NORMAL)) exit (iosb[0]);
}

spawn (command, sysoutput, wait)
    char *command, *sysoutput;
    int wait;
{
    $DESCRIPTOR (comdescr, command);
    $DESCRIPTOR (outdescr, sysoutput);
    int status, event_flag = 1;
    struct procinfo *process, *procptr;
    extern struct procinfo *proclist;

    comdescr.dsc$w_length = strlen (command);
    outdescr.dsc$w_length = strlen (sysoutput);

    process = (struct procinfo *) calloc (1, sizeof (struct procinfo));
    process->output_ppf = sysoutput;

    if (!((status = lib$spawn (&comdescr, 0, &outdescr, &1, 0,
			       &process->id, 0, (wait ? &event_flag : 0),
			       (wait ? 0 : &terminator),
			       (wait ? 0 : process)
			      )) & SS$_NORMAL))
	exit (status);

    if (proclist == 0)
	proclist = process;
    else
    {
	procptr = proclist;
	while (procptr->next != 0) procptr = procptr->next;
	procptr->next = process;
    }

    if (wait)
	if (!((status = sys$waitfr (event_flag)) & SS$_NORMAL))
	    exit (status);
}

killproc(rsnptr)
int *rsnptr;
{
    extern struct procinfo *proclist;
    struct procinfo *procptr;
    int status;

    if (!(procptr = proclist)) exit (*rsnptr);
    do
    {
	status = sys$delprc (&procptr->id, 0);
	if (!(status & SS$_NORMAL) && (status != SS$_NONEXPR))
	    exit (status);
    }
    while (procptr = procptr->next);
    exit (*rsnptr);
}
