@Comment(-*-SCRIBE-*-)
@Comment(SCRIBE Text Formatter Input for the KERMIT Protocol Manual)

@Make<Manual>
@Comment(Use /draft:F on command line to produce .LPT file w/cc in column 1)

@Style<Justification On, Hyphenation On, WidestBlank 1.4, Spacing 1,
        Spread 1, Indent 0, HyphenBreak Off, SingleSided>
@Modify<IndexEnv, Columns 2, ColumnMargin 0.5inch, LineWidth 3inch, Boxed>

@Comment(Printing Device Dependencies)

@Define<Q,FaceCode R>
@Case<Device,
 LPT="@use(Auxfile='PROTLP.AUX')
    @Case[Draft,F=`@Style(CarriageControl=FORTRAN)']
    @Style(linewidth 79)
    @Modify(ProgramExample,LongLines Keep)",
 Printronix="@use(AuxFile='PROTPX.AUX')
    @Style(LineWidth 74, PaperLength 11 inches,BottomMargin 5 spacings)
    @Modify(ProgramExample,LongLines Keep)",
 PagedFile="@use(AuxFile='PROTPF.AUX')
    @Style(LineWidth 79, PaperLength 10.8 inches)
    @Style(TopMargin 3 spacings,BottomMargin 6 Spacings)
    @Modify(ProgramExample,LongLines Keep)",
 Diablo="@Use(Auxfile='PROTDI.AUX')
    @TypeWheel(Titan 10)",
 Imprint10="@Use(AuxFile='PROTIM.AUX')
    @Define<Q,FaceCode U>
    @Modify<Insert,Spacing 1.2>
    @Modify<Verbatim,FaceCode U>
    @Modify<Example,FaceCode U>
    @Modify<Itemize,Spread 0.8>
    @Style<FontFamily SmallRoman12,Spacing 1.6,SingleSided>",
 X9700="@Use<AuxFile='PROTX9.AUX'>
    @Define<Q,FaceCode U>
    @Modify(Verbatim,FaceCode U)
    @Modify<Example,FaceCode U>
    @Style<FontFamily Univers10,DoubleSided>
    @Use<HyphenDic=`KER:MANUAL.HYP'>"
 >

@Comment(Set desired spacing around various environments)

@Modify<Quotation,Indentation 0, Above 1, Below 1>
@Modify<Example, Above 1, Below 1, Blanklines Hinge>
@Modify<Verbatim,Leftmargin 0>
@Modify<ProgramExample, Above 1, Below 1,Leftmargin 0, BlankLines Hinge>
@Modify<Description,Above 1,Below 1>
@Modify<Enumerate,Above 1,Below 1>

@Comment(Set spacing and paging requirements for chapter & section headings)

@Modify(Hdx,Above 2,Below 1,Need 8)
@Modify(Hd0,Above 2,Below 1,Need 8)
@Modify(Hd2,Above 2,Below 1,Need 8)
@Modify(Hd3,Above 2,Below 1,Need 8)
@Modify(Hd4,Above 2,Below 1,Need 8)

@String(What="Manual")

@Comment(Start the document off with a titlepage)

@Begin(TitlePage,Initialize "@BlankSpace(2.5inches)",sink 0)
@MajorHeading(KERMIT PROTOCOL MANUAL)

@i<Fourth Edition>




Frank da Cruz, Bill Catchings@foot<Present address: Lehman Bros Kuhn Loeb, NYC>

Columbia University Center for Computing Activities
New York, New York  10027


@value<date>






Copyright (C) 1981,1982,1983
Trustees of Columbia University in the City of New York

@i<Permission is granted to any individual or institution to copy or
use this document and the programs described in it, except for
explicitly commercial purposes.>

@end(titlepage)

@case<device,x9700="@pageheading(Center `',immediate)
  @blankpage(1)
  @set(page=-2)">
@pageheading<center"- @value[page] -">

@PrefaceSection<Preface to the Fourth Edition>

The fourth edition of the KERMIT Protocol Manual incorporates some new ideas
that grew from our experience in attempting to implement some of the features
described in earlier editions, particularly user/@|server functions.  These
include a mechanism to allow batch transfers to be interrupted gracefully
for either the current file or the entire batch of files; a "capability mask";
a protocol extension for passing file attributes.  In addition, numbers are now
written in decimal notation, rather than octal, which was confusing to many
readers.  Also, several incompatible changes were made in minor areas where no
attempts at an implementation had yet been made; these include:
@Begin(Itemize)
The format and interpretation of the operands to the server commands.

Usurpation of the reserved fields 10-11 of the Send-Init packet, and addition
of new reserved fields.
@End(Itemize)
Most of the remaining material has been rewritten and reorganized, and much new
material added, including a section on the recommonded vocabulary for
documentation and commands.

The previous edition of the Protocol Manual attempted to define "protocol
version 3"; this edition abandons that concept.  Since KERMIT development is
an unorganized, disorderly, distributed enterprise, no requirement can be
imposed on KERMIT implementors to include a certain set of capabilities in
their implementations.  Rather, in this edition we attempt to define the basic
functionality of KERMIT, and then describe various optional functions.

The key principle is that any implementation of KERMIT should work with any
other, no matter how advanced the one or how primitive the other.  The
capabily mask and other Send-Init fields attempt to promote this principle.

@begin[i]
Before attempting to write a new Kermit program, be sure to check with the
authors to make sure that no one else is working on the same thing, and that
you have the latest information.
@end[i]

The Columbia University DEC-20 implementation of KERMIT should be considered
the definitive implementation of the protocol.  The true test of any new Kermit
is whether it can talk to Columbia DEC-20 Kermit.

KERMIT is distributed from Columbia University on magnetic tape.  Complete
ordering instructions are in the @i<Kermit Users Guide>.

@begin<verbatim, leftmargin +4, facecode R>
KERMIT Distribution
Columbia University Center for Computing Activities
7th Floor, Watson Laboratory
612 West 115th Street
New York, NY  10025
@end<verbatim>

No warranty of the software nor of the accuracy of the documentation
surrounding it is expressed or implied, and neither the authors nor Columbia
University acknowledge any liability resulting from program or documentation
errors.

@Chapter<Introduction>

 This manual describes the @Index[KERMIT] KERMIT @Index[Protocol]
protocol.  It is assumed that you understand the purpose and operation of the
Kermit file transfer facility, described in the @i<Kermit Users Guide>, and
basic terminology of data communications and computer programming.

@Section<Background>

The KERMIT file transfer protocol is intended for use in an environment where
there may be a diverse mixture of computers -- micros, personal computers,
workstations, laboratory computers, timesharing systems -- from a variety of
manufacturers.  All these systems need have in common is the ability to
communicate in ASCII over ordinary serial telecommunication lines.

KERMIT was originally designed at Columbia to meet the need for file transfer
between our DECSYSTEM-20 and IBM 370-@|series mainframes and various
microcomputers.  It turned out that the diverse characteristics of these three
kinds of systems resulted in a design that was general enough to fit almost any
system.  The IBM mainframe, in particular, strains most common assumptions
about how computers communicate.

@Section<Overview>

The KERMIT protocol is specifically designed for character-@|oriented
transmission over serial telecommunication lines.  The design allows for the
restrictions and peculiarities of the medium and the requirements of diverse
operating environments -- buffering, duplex, parity, character set, file
organization, etc.  The protocol is carried out by KERMIT programs on each end
of the serial connection sending "packets" back and forth; the sender sends
file names, file contents, and control information; the receiver acknowledges
(positively or negatively) each packet.

The packets have a layered design, in keeping with the ANSI and ISO
philosophies, with the outermost fields used by the data link layer to verify
data integrity, the next by the session layer to verify continuity, and the
data itself at the application level.

Connections between systems are established by the ordinary user.  In a typical
case, the user runs KERMIT on a microcomputer, enters terminal emulation,
connects to a remote host computer (perhaps by dialing up), logs in, runs
KERMIT on the remote host, and then issues commands to that KERMIT to start
a file transfer, "escapes" back to the micro, and issues commands to that
KERMIT to start its side of the file transfer.  Files may be transferred singly
or in groups.

Basic KERMIT provides only file transfer, and that is provided for
@i<sequential files only>@index(Sequential Files), though the protocol attempts
to allow for various types of sequential files.  Microcomputer implementations
of KERMIT are also expected to provide terminal emulation, to facilitate the
initial connection.

More advanced implementations simplify the "user interface" somewhat by
allowing the KERMIT on the remote host to run as a "server", which can transfer
files in either direction upon command from the local "user" Kermit.  The
server can also provide additional functionality, such as file management,
messages, mail, and so forth.  Other optional features also exist, including a
variety of block check types, a mechanism for passing 8-bit data through a
7-bit communication link, a way to compressing a repeated sequence of
characters, and so forth.

As local area networks become more popular, inexpensive, and standardized, the
demand for KERMIT and similar protocols may dwindle, but will never wither away
entirely.  Unlike hardwired networks, KERMIT gives the ordinary user the power
to establish reliable error-@|free connections between @i<any two> computers;
this may always be necessary for one-@|shot or long-@|haul connections.

@Chapter<Definitions>

@Section<General Terminology>

@Index<TTY>
@u<TTY>: This is the term commonly used for a device which is connected to a
computer over an EIA RS-232 serial telecommunication line.  This device is most
commonly an ASCII terminal, but it may be a microcomputer or even a large
multi-@|user computer emulating an ASCII terminal.  Most computers provide
hardware (RS-232 connectors and UARTs) and software (device drivers) to support
TTY connections; this is what allows TTY-@|oriented file transfer protocols
like KERMIT possible on almost any system at little or no cost.

@Index[Local]
@u<LOCAL>: When two machines are connected, the LOCAL machine is the one which
you interact with directly, and which is in control of the terminal.  The
"local Kermit" is the one that runs on the local machine.  A local Kermit
always communicates over an external device (the micro's communication port, an
assigned TTY line, etc).

@index<Remote>
@u<REMOTE>: The REMOTE machine is the one on the far side of the connection,
which you must interact with "through" the local machine.  The "remote Kermit"
runs on the remote machine.  A remote Kermit usually communicates over its own
"console", "controlling terminal", or "standard i/o" device.

@index<Host>
@u<HOST>: Another word for "computer", usually meaning a computer that can
provide a home for multiple users or applications.  This term should be avoided
in KERMIT lore, unless preceded immediately by LOCAL or REMOTE.

@index<Server>
@u<SERVER>: An implementation of remote Kermit that can accept commands 
from a local Kermit program, instead of directly from the user.

@index<User>
@u<USER>: In addition to its usual use to denote the person using a system or
program, "user" will also be used refer to the local Kermit program, when the
remote Kermit is a server.

@Section<Numbers>

All @u<numbers> in the following text are expressed in decimal
(base 10) notation unless otherwise specified.   

@Index<Bit Positions>
Numbers are also referred to in terms of their bit positions in a computer
word.  Since KERMIT may be implemented on computers with various word sizes, we
start numbering the bits from the "right" -- bit 0 is the least significant.
Bits 0-5 are the 6 least significant bits; if they were all set to one, the
value would be 63.

@Index<8th Bit>
A special quirk in terminology, however, refers to the high order bit of a
character as it is transmitted on the communication line, as the "8th bit".
More properly, it is bit 7.  References to the "8th bit" generally are with
regard to that bit which ASCII transmission sets aside for use as a parity bit.
KERMIT concerns itself with whether this bit can be usurped for the
transmission of data, and if not, it may resort to "8th-bit prefixing".

@Section<Character Set>

All @u<characters> are in ASCII @index<ASCII> (American national Standard Code
for Information Interchange) representation, ANSI standard X3.4-1968.  All
implementations of KERMIT transmit and receive characters only in ASCII.

@ux<ASCII character mnemonics:>
 @Begin<Description,Spread 0,leftmargin +8,indent -8>
NUL@\Null, idle, ASCII character 0.

SOH@\Start-of-header, ASCII character 1 (Control-A).

SP@\Space, blank, ASCII 32.

CR@\Carriage return, ASCII 13 (Control-M).

LF@\Linefeed, ASCII 10 (Control-J).

CRLF@\A carriage-@|return linefeed sequence.

DEL@\Delete, rubout, ASCII 127.
@end<description>

A @ux<control character> is considered to be any ASCII character in the
range 0 through 31, or the DEL character (127).  In this document, control
characters are written in several ways:
@Begin(Description,leftmargin +8,indent -8)
Control-A@\This denotes ASCII character 1, commonly referred to as "Control-A".
Control-B is ASCII character 2, and so forth.

CTRL-A@\This is a common abbreviation for "Control-A".  A control character is
generally typed at a computer terminal by holding down the key marked CTRL and
pressing the corresponding alphabetic character, in this case "A".

@q<^A>@\"Upparow" notation for CTRL-A.  Many computer systems "echo" control
characters in this fashion.
@End(Description) 

A @ux<printable ASCII character> is considered to be any character in the
range 32 (SP) through 126 (tilde).

@Section<Conversion Functions>

Several conversion functions are useful in the description of the protocol
and in the program example.  The machine that Kermit runs on need operate only
on integer data; these are functions that operate upon the numeric value of
single ASCII characters.
 @begin<description>
@Index[char(x)]
@q<char(x) = x+32>@\Transforms the integer @i<x>, which is assumed to lie in
the range 0 to 94, into a printable ASCII character; 0 becomes SP, 1 becomes
"!", etc.

@Index[unchar(x)]
@q<unchar(x) = x-32>@\Transforms the character @i<x>, which is assumed to be in
the printable range (SP through tilde), into an integer in the range 0 to 94.

@Index[ctl(x)]
 @q<ctl(x) = x XOR 64>@\Maps between control characters and their printable
representations, preserving the high-@|order bit@foot{The high order bit is
normally the parity bit, but Kermit uses this bit as data in order to do 8-bit
transmission for binary files when the systems permit.}.  If @i<x> is a control
character, then
 @example<x = ctl(ctl(x))>
 that is, the same function is used to controllify and uncontrollify.  The
argument is assumed to be a true control character (0 to 31, or 127), or the
result of applying @c<ctl> to a true control character (i.e. 63 to 95).  The
transformation is a mnemonic one -- @q(^A) becomes A and vice versa.
@end<description>

@Section<Protocol Jargon>

@Index[Packet]
A @u<Packet> is a clearly delimited string of characters, comprised "control
fields" nested around data; the control fields allow a KERMIT program to
determine whether the data has been transmitted correctly and completely.
A packet is the smallest unit of transmission in the KERMIT protocol.

@Index[ACK]
@u<ACK> stands for "Acknowledge".  An ACK is a packet that is sent to
acknowledge receipt of another packet.  Not to be confused with the ASCII
character ACK.

@Index[NAK]
@u<NAK> stands for "Negative Acknowledge".  A NAK is a packet sent to say that
a corrupted or incomplete packet was received, the wrong packet was received,
or an expected packet was not received.  Not to be confused with the ASCII
character NAK.

@Chapter<System Requirements>

The KERMIT protocol requires that@q<:>
@begin<itemize>
The host can send and receive characters using 7- or 8-bit ASCII encoding over
a EIA RS-232 physical connection, either hardwired or dialup.

All printable ASCII characters are acceptable as input to the host and will not
be transformed in any way@foot<If they are translated to another character set,
like EBCDIC@index(EBCDIC), the KERMIT program must be able to reconstruct the
packet as it appeared on the communication line, before transformation.>.
Similarly, any intervening network or communications equipment (TELENET,
terminal concentrators, port selectors, etc) must not transform or swallow any
printable ASCII characters.


@index<SOH>
A single ASCII control character can pass from one system to the other
without transformation.  This character is used for packet synchronization.
The character is normally Control-A (SOH, ASCII 1), but can be redefined.

@IndexEntry[Key="Line Terminator (see End-Of-Line)",
  Text="Line Terminator (see End-Of-Line)"]
@Index[End-Of-Line (EOL)]
If a host requires a line terminator for terminal input, that terminator
must be a single ASCII control character, such as CR or LF, distinct from the
packet synchronization character.

@Index[Remote]@Index[Binary Mode]@Index[Raw Mode]
 When using a job's controlling terminal for file transfer, the system must
allow the KERMIT program to set the terminal to half duplex, infinite width (no
"wraparound" or CRLF insertion by the operating system), and no "formatting" of
incoming or outgoing characters (for instance, raising lowercase letters to
uppercase, transforming control characters to printable sequences, etc).  In
short, the terminal must be put in "binary" or "raw" mode, and, hopefully,
restored afterwards to normal operation.

The host's terminal input processor should be capable of receiving a single
burst of 40 to 100 characters at normal transmission speeds.  This is the
typical size of packet.

@Index[Padding]
If a host requires padding, the padding character must be a control character
(including DEL).
@end<itemize>

KERMIT does @i<not> require:
@begin<itemize>
@Index<Baud>
That the connection run at any particular baud rate.

@index<Flow Control>
That the system can do @Index[XON/XOFF] XON/XOFF or any other kind of flow
control.  System- or hardware-@|level flow control can help, but it's not
necessary.  See section @ref<-flow>.

@Index[Duplex]@Index[Full Duplex]@Index[Half Duplex]
That the system is capable of full duplex operation.  Any mixture of half and
full duplex systems is supported.

@Index<Binary Files>
That the system can transmit or receive 8-bit bytes.  KERMIT will take
advantage of 8-bit connections to send binary files; if an 8-bit connection is
not possible, then binary files may be sent using an optional prefix encoding.
 @end<itemize>

@Chapter<Printable Text versus Binary Data>

@Index[Records]@Index[Logical Records] @Index[Printable Files]
@Index[Binary Files] @Index[Text Files]
For transmission between unlike systems, files must be assigned to
either of two catagories: @i<printable text> or @i<binary>.

A printable text file is one that can make sense on an unlike system -- a
document, program source, textual data, etc.  A binary file is one that will
not (and probably can not) make sense on an unlike system -- an executable
program, numbers stored in internal format, etc.  On systems with 8-bit bytes,
printable ASCII files will have the high order bit of each byte set to
zero@foot<There are some exceptions, such as systems that store text files in
so-@|called "negative ASCII", or text files produced by word processors that
use the high order bit to indicate underline or boldface attributes.>
(since ASCII is a 7-bit code) whereas binary files will use the high order bit
of each byte for data, in which case its value can vary from byte to byte.

Since computers can't be expected to distinguish a printable file from a binary
file -- especially one originating from an unlike system -- the user may have
to give an explicit command to Kermit to tell it whether to perform these
conversions.

@Section<Printable Text Files>

For printable text files to be transferred in a useful fashion, there must be a
standard way to represent them during transmission.  KERMIT's standard is
simple: ASCII characters, with "logical records" (lines) delimited by CRLFs.
It is the responsibility of systems that do not store printable files in this
fashion to perform the necessary conversions upon input and output.  For
instance, IBM mainframes might strip trailing blanks on output and add them
back on input; UNIX would prepend a CR to its normal record terminator, LF,
upon output and discard it upon input.

@Section<Binary Files>

Binary files are transmitted as though they were a sequence of characters.  The
difference from printable files is that the status of the "8th bit" must be
preserved.  When binary files are transmitted to an unlike system, the main
objective is that they can be brought back to the original system (or one
like it) intact; no special conversions are necessary during transmission,
except to make the data fit the transmission medium.

For binary files, eight bit character transmission is permissible as
long as the two Kermit programs involved can control the value of the parity
bit, and no intervening communications equipment will change its value.  In
that case, the 8th bit of a transmitted character will match that of the
original data byte, after control-@|quoting has been done.  When one or both
sides cannot control the parity bit, a special prefix character may be
prepended, as described below.

Systems that do not store binary data in 8-bit bytes, or whose word size is not
a multiple of 8, may make special provisions for "image mode" transfer of
binary files.  This may be done within the basic protocol by having the two
sides implicitly agree upon a scheme for packing the data into 7-
or 8-bit ASCII characters, or else the more flexible (but optional) file
attributes feature may be used.

@Chapter<Packet Format>

@Index[Packet]
The KERMIT protocol is built around exchange of packets of the
following format:
@begin<example>
+------+-----------+-----------+------+------------+-------+
| MARK | char(LEN) | char(SEQ) | TYPE |    DATA    | CHECK |
+------+-----------+-----------+------+------------+-------+
@end<example>
where all fields consist of ASCII characters.  The fields are:
 @begin<description,leftmargin +8,indent -8>
@u<MARK>@\The synchronization character that marks the beginning of the
packet.  This should normally be CTRL-A, but may be redefined.

@u<LEN>@\The number of ASCII characters within the packet that follow this
field, in other words the packet length minus two.  Since this number is
transformed to a single character via the char function, packet character
counts of 0 to 94 (decimal) are permitted, and 96 (decimal) is the maximum
total packet length.  The length does not include end-@|of-@|line or padding
characters, which are outside the packet and are strictly for the benefit of
the operating system, but it does include the block check characters.

@u<SEQ>@\The packet sequence number, modulo 64, ranging from 0 to 63.
Sequence numbers "wrap around" to 0 after each group of 64 packets.

@u<TYPE>@\The packet type, a single ASCII character.  The following packet
types are required:
 @begin<description,spread 0, leftmargin +4, indent -4>
D@\Data packet

Y@\Acknowledge (ACK)

N@\Negative acknowledge (NAK)

S@\Send initiate (exchange parameters)

B@\Break transmission (EOT)

F@\File header

Z@\End of file (EOF)

E@\Error
@end<description>

@Index[Control Characters]
@u<DATA>@\The "contents" of the packet, if any contents are required in the
given type of packet, interpreted according to the packet type.  Control
characters are preceded by a special prefix character, normally "@q<#>", and
"uncontrollified" via @q<ctl()>.  A prefixed sequence may not be broken across
packets.  Logical records in printable files are delimited with CRLFs, suitably
prefixed (e.g. "@q<#M#J>").  Any prefix characters are included in the count.
Optional encoding for 8-bit data and repeated characters is described later.

@begin[multiple]
@u<CHECK>@\@Index[Checksum]@Index<Block Check>
A block check on the characters in the packet
between, but not including, the mark and the block check itself.  The check for
each packet is computed by both hosts, and must agree if a packet is to be
accepted.  A single-@|character arithmetic checksum is the normal and required
block check.  Only six bits of the arithmetic sum are included.  In order that
all the bits of each data character contribute to this quantity, bits 6 and 7
of the final value are added to the quantity formed by bits 0-5.  Thus if @i<s>
is the arithmetic sum of the ASCII characters, then
@example[@i<check> = char((@i<s> + ((@i<s> AND 300)/100)) AND 77)]
This is the default block check, and all Kermits must be capable of performing
it.  Other optional block check types are described later.

The block check is based on the ASCII values of the characters in the packet.
Non-@|ASCII systems must translate to ASCII before performing the block check
calculation.
@end[multiple]
@end<description>

@Index[End-Of-Line (EOL)]
Any line terminator that is required by the system may be appended to the
packet; this is carriage return (ASCII 15) by default.  Line terminators are
not considered part of the packet, and are not accounted for in the count or
checksum.  Terminators are not necessary to the protocol, and are invisible to
it, as are any characters that may appear between packets.  If a host cannot do
single character input from a TTY line, then a terminator will be required for
that host.  The terminator can be specified in the initial connection protocol.

@Chapter<Initial Connection>

@Index[Initial Connection]@Index<Send-Init>
Initial connection occurs when the user has started up a Kermit program on both
ends of the physical connection.  One Kermit has been directed (in one way or
another) to send a file, and the other to receive it.  

The receiving Kermit waits for a "Send-Init" packet from the
sending Kermit.  It doesn't matter whether the sending Kermit is started before
or after the receiving Kermit (if before, the Send-Init packet should be
retransmitted periodically until the receiving Kermit acknowledges it).  The
data field of the Send-Init packet is optional; trailing fields can be
omitted (or left blank, i.e. contain a space) to accept or specify default
values.

It is important to note that newly invented fields are added at the right, so
that old KERMIT programs that do not have code to handle the new fields will
act as if they were not there.  For this reason, the default value for any
field, indicated by blank, should result in the behavior that occurred before
the field was added.

@begin<example,leftmargin +2>
 1      2      3      4      5      6      7      8      9      10...
+------+------+------+------+------+------+------+------+------+-------------
| MAXL | TIME | NPAD | PADC | EOL  | QCTL | QBIN | CHKT | REPT | Capabilities 
+------+------+------+------+------+------+------+------+------+-------------
@end<example>
The fields are as follows (the first and second person "I" and "you" are
used to distinguish the two sides).  Fields are encoded printably using the
@q<char()> function unless indicated otherwise.
@begin<description,leftmargin +10,indent -10>
1. @u<MAXL>@\The maximum length packet I want to receive, a number up to
94 (decimal).  You respond with the maximum you want me to send. 
This allows systems to adjust to each other's buffer sizes, or to the condition
of the transmission medium.

2. @u<TIME>@\The number of seconds after which I want you to time me out while
waiting for a packet from me.  You respond with the
amount of time I should wait for packets from you.  This allows the two sides
to accommodate to different line speeds or other factors that could cause
timing problems.

3. @u<NPAD>@\The number of padding characters I want to precede each incoming
packet; you respond in kind.  Padding may be necessary
for a half duplex system that requires some time to change the direction of
transmission, although in practice this situation is more commonly handled by a
"handshake" mechanism.

4. @u<PADC>@\The control character I need for padding, if any, transformed by
@q<ctl()> (@i<not> @q<char()>) to make it printable.  You respond in kind.
Normally NUL (ASCII 0), some systems use DEL (ASCII 127).  This field is
ignored if the value NPAD is zero.

5. @u<EOL>@\The character I need to terminate an incoming packet, if any.  You
respond in kind.  Most systems that require a line terminator for terminal
input accept carriage return for this purpose.

6. @u<QCTL>@\(verbatim) The printable ASCII character I will use to quote
control characters, normally "@q<#>".  You respond with the one you will use.

7. @u<QBIN>@\(verbatim) The printable ASCII character I want to use to quote
characters which have the 8th bit set, for transmitting binary files when the
parity bit cannot be used for data.  Since this kind of quoting increases both
processor and transmission overhead, it is normally to be avoided.  If used,
the quote character must be in the range 33-62 ("@q<!>" through "@q(>)")
or 96-126 ("@q<`>" through "@q<~>"), but different from
the control-@|quoting character.  This field is interpreted as follows:
@begin<description,leftmargin +4,indent -4,spread 0>
@q<Y>@\I agree to 8-bit quoting if you request it.

@q<N>@\I will not do 8-bit quoting.

@q<&>@\(or any other character in the range 33-62 or 96-126 besides Y and N)
I want to do 8-bit quoting using this character (it will be done if the other
Kermit puts a @q<Y> in this field).  The recommended 8th-bit quoting prefix
character is "@q<&>".

@i<Anything Else>: 8-bit quoting will not be done.
@end<description>

8. @u<CHKT>@\Check Type, the method for detecting errors.  "1" for
single-@|character checksum (the normal and required method), "2" for
two-@|character checksum (optional), "3" for three-@|character CRC-@|CCITT
(optional).  If your response agrees, the designated method will be used;
otherwise the single-@|character checksum will be used.

9. @u<REPT>@\The prefix character I will use to indicate a repeated character.
This can be any printable character in the range 33-62 or 96-126, but different
from the control and 8th-bit prefixes.  SP (32) denotes no repeat count
processing is to be done.  Tilde ("@q<~>") is the recommended and normal repeat
prefix.  If you don't respond identically, repeat counts will not be done.
Groups of at least 3 or 4 identical characters may be transmitted more
efficiently using a repeat count, though an individual implementation may wish
to set a different threshhold.

10-?. @Index[Capabilies]@u[Capabilities]@\A bit mask, in which each bit
position corresponds to a capability of KERMIT, and is set to 1 if that
capability is 
present, or 0 if it is not.  Each character contains a 6-bit field (transformed
by @q<CHAR()>), whose low order bit is set to 1 if another capability byte
follows, and to 0 in the last capability byte.  The capabilities defined so far
are:
@Begin(Description,leftmargin +6,indent -4,spread 0)
@q<#1>@\Ability to time out.

@q<#2>@\Ability to accept server commands.

@q<#3>@\Ability to accept "A" packets (file attributes)
@End(Description) 
The capability byte as defined so far would then look like:
@begin<example,leftmargin +2>
 bit5 bit4 bit3 bit2 bit1 bit0
+----+----+----+----+----+----+
| #1 | #2 | #3 | -- | -- |  0 |
+----+----+----+----+----+----+
@end<example>
If all these capabilities were "on", the value of the byte would be 70
(octal).  When capabilities 5 and 6 are added, the capability mask will look
like this:
@begin<example,leftmargin +2>
 bit5 bit4 bit3 bit2 bit1 bit0     bit5 bit4 bit3 bit2 bit1 bit0
+----+----+----+----+----+----+    +----+----+----+----+----+----+
| #1 | #2 | #3 | #4 | #5 |  1 |    | #6 | -- | -- | -- | -- |  0 |
+----+----+----+----+----+----+    +----+----+----+----+----+----+
@end<example>

@i<Next 4:>@Index[Reserved Fields]@ @ux[Reserved Fields]@\Sites that wish to
add their own parameters to the initial connection negotiation must start at
the 5th field after the last capability byte.  Any intervening fields may be
left blank (that is, they may contain the space character).  These fields are
reserved for future use by the standard KERMIT protocol.
 @end<description>
@i<The control, 8th-bit, and repeat prefixes must be distinct.>

The receiving Kermit responds with an ACK ("Y") packet in the same format to
indicate its own preferences, options, and parameters.  The ACK need not
contain the same number of fields as the the Send-Init.  From that point, the
two KERMIT programs are "configured" to communicate with each other for the
remainder of the transaction.  In the case of 8th-bit quoting, one side must
specify the character to be used, and the other must agree with a "Y" in the
same field, but the order in which this occurs does not matter.  Similarly for
checksums -- if one side requests 2 character checksums and the other side
responds with a "1" or with nothing at all, then single-@|character checksums
will be done, since not all implementations can be expected to do 2-@|character
checksums or CRCs.  And for repeat counts; if the repeat field of the send-init
and the ACK do not agree, repeat processing will not be done.

All Send-Init fields are optional.  The data field may be left totally empty.
Similarly, intervening fields may be defaulted by setting them to blank.
Kermit implementations should know what to do in these cases, namely apply
appropriate defaults.  The defaults should be:
@begin<description,leftmargin +12, indent -6, spread 0>
@label<-defaults>
MAXL:@\80

NPAD:@\0, no padding

PADC:@\0 (NUL)

EOL:@\CR (carriage return)

QCTL:@\the character "#"

QBIN:@\none, don't do 8-bit quoting

CHKT:@\"1", single-character checksum

REPT:@\No repeat count processing

MASK:@\All zeros (no special capabilities)
@end<description>

Note that there are no prolonged negotiations in the initial connection
sequence -- there is one Send-Init and one ACK in reply.  Everything must
be settled in this exchange.

The very first Send-Init may not get through if the sending Kermit makes
wrong assumptions about the receiving host.  For instance, the receiving host
may require certain parity, some padding, or a special end of line character in
order to read the Send-Init packet.  For this reason, there should be
a way for the user the user to specify whatever may be necessary to get the
first packet through.

A parity field is not provided in the Send-Init packet because it could not be
of use.  If the sender requires a certain kind of parity, it will also be
sending it.  If the receiver does not know this in advance, i.e. @i<before>
getting the Send-Init, it will not be able to read the Send-Init packet.

@Chapter<File Transfer>

@Index<Transaction>
The file transfer protocol is defined over a @i<transaction>.  A
transaction@index<Transaction> is defined as an exchange of packets
beginning with a Send-Init (S) packet, and ending with a
Break Transmission (B) or Error (E) packet@foot<A transaction should also be
considered terminated when one side or the other has stopped without sending an
Error packet.>, and may include the transfer of one
or more files, all in the same direction.  The following sequence
characterizes basic Kermit operation; the @i<sender> is the machine that is
sending files; the @i<receiver> is the machine receiving the files.
@Begin(Enumerate)
The sender transmits a Send-@|Initiate (S) packet to specify
its parameters (packet length, timeout, etc).

The receiver sends an ACK (Y) packet, with its own parameters in the data
field.

The sender transmits a File-Header (F) packet, which contains the file's name
in the data field.  The receiver ACKs the F packet, with no data in the data
field of the ACK (optionally, it may contain the name under which the receiver
will store the file under).
@tag(-fpacket)

The sender sends the contents of the file, in Data (D) packets.  Each D packet
must be acknowledged before the next one is sent.

When all the file data has been sent, the sender sends an End-@|Of-@|File (Z)
packet.  The receiver ACKs it.

If there is another file to be sent, the process is repeated, beginning at step
@ref<-fpacket>.

When no more files remain to be sent, the sender transmits an
End-@|Of-@|Transmission (B) packet.  The receiver ACKs it.  This ends the
transaction, and closes the logical connection (the physical connection remains
open).
@end<enumerate>
Each packet has a sequence number, starting with 0.  The acknowledgment (ACK or
NAK) for a packet has the same packet number as the packet it is acknowledging.
Once an acknowledgment is successfully received, the sender increments the
packet number by one, modulo 64.

If the sender is remote, it waits for a certain amount of time (somewhere in
the 5-30 second range) before transmitting the Send-Init, to give the user time
to escape back to the local KERMIT and tell it to receive files.

@Section<Timeouts, NAKs, and Retries>

If a KERMIT program is capable of setting a timer interrupt, or setting a time
limit on an input request, it should do so whenever attempting to read a packet
from the communication line, whether sending or receiving files.
Having read a packet, it should turn off the timer.

If the sender times out waiting for an acknowledgement, it should send the same
packet again, repeating the process a certain number of times up to a retry
limit, or until an acknowledgement is received.  If the receiver times out
waiting for a packet, it can send either a NAK packet for the expected packet
or another ACK for the last packet it got.

If a packet from the sender is garbled or lost in transmission (the latter is
detected when the sequence number increases by more than 1, modulo 64, the
former by a bad checksum), the receiver sends a NAK for the garbled or missing
packet.  If an ACK or a NAK from the receiver is garbled or lost, the sender
ignores it; in that case, one side or the other will time out and retransmit.

A retry count is maintained, and there is a retry threshold, normally set at
about 5.  Whenever a packet is resent -- because of a timeout, or because it
was NAK'd -- the counter is incremented.  When it reaches the threshold, the
transaction is terminated.

If neither side is capable of timing out, a facility for manual intervention
must be available on the local KERMIT.  Typically, this will work by sampling
the keyboard (console) periodically; if input, such as a CR, appears, then the
same action is taken as if a timeout had occurred.  The local KERMIT keeps a
running display of the packet number or byte count on the screen to allow the
user to detect when traffic has stopped.  At this point, manual intervention
should break the deadlock.

Shared systems which can become sluggish when heavily used should adjust their
own timeout intervals on a per-@|packet basis, based on the system load, so
that file transfers won't fail simply because the system was too slow.

@section<Errors>

@Index<Errors>@Index<Fatal Errors>
During file transfer, the sender may encounter an i/o error on the disk, or the
receiver may attempt to write to a full or write-@|protected device.  Any
condition that will prevent successful transmission of the file is called a
"fatal error".  Fatal errors should be detected, and the transfer shut down
gracefully, with the pertinent information provided to the user.  Error packets
provide a mechanism to do this.

If a fatal error takes place on either the sending or receiving side, the side
which encountered the error should send an Error (E) packet.  The E packet
contains a brief English error message in the data field.  Both the sender and
receiver should be prepared to receive an Error packet at any time during the
transaction.  Both the sender and receiver of the Error packet should halt, or
go back into into user command mode.  The side that is local should print the
error message on the screen.

@section<Heuristics>

A few heuristics are useful:
@begin<enumerate>
A NAK for the current packet is equivalent to an ACK for the previous packet.
This covers the common situation in which a packet is successfully received,
and then ACK'd, but the ACK is lost.  The ACKing side then times out waiting
for the next packet and NAKs it.  The side that receives a NAK for packet
@i<n+1> while waiting for an ACK for packet @i<n> simply sends packet @i<n+1>.

If packet @i<n> arrives more than once, simply ACK it and discard it.  This
can happen when the first ACK was lost.  Resending the ACK is necessary @i<and>
sufficient -- don't write the packet out to the file again!

When opening a connection, discard the contents of the line's input buffer
before reading or sending the first packet.  This is especially important if
the other side is in receive mode  (or acting as a server), in which case
it may have been sending out periodic NAKs for your expected SEND-@|INIT or
command packet.  If you don't do this, you may find that there are sufficient
NAKs to prevent the transfer -- you send a Send-Init, read the response, which
is an old NAK, so you send another Send-Init, read the next old NAK, and so
forth, up to the retransmission limit, and give up before getting to the ACKs
that are waiting in line behind all the old NAKs.  If the number of NAKs is
below the cutoff, then each packet may be transmitted multiply.

Similarly, after reading a packet (successfully or not), you should clear the
input buffer.  There should be nothing there for you anyway, since the other
side must normally wait for you to send your packet in response.  Failure to
clear the buffer could result in propogation of the repetition of a packet
caused by stacked-up NAKs.
@end<enumerate>


@Section<File Names>
@Index<File Names>
The syntax for file names can vary widely from system to system.  To avoid
problems, it is suggested that filenames be represented in the File Header (F)
packet in a "normal form", by default (that is, there should be an option to
override such conversions). 

@Begin(Enumerate)
@Index<Normal Form for File Names>
Delete all pathnames and attributes from the file specification.  The file
header packet should not contain directory or device names; if it does, it may
cause the recipient to try to store the file in an inaccessible or nonexistent
area, or it may result in a very strange filename.

After stripping any pathname, convert the remainder of the file specification
to the form "@i<name>@q<.>@i<type>", with no restriction on length (except that
it fit in the data field of the F packet), and: @Begin(Enumerate, spread 0)
Include no more than one dot.

Use digits, uppercase letters only in @i"name" and @i"type".
@End(Enumerate)
@End(Enumerate)
Special characters like @q<"$">, @q<"_">, @q<"-">, @q<"&">, and so
forth should be disallowed, since they're sure to cause problems on
one system or another.

The recipient, of course, cannot depend upon the sender to follow this
convention, and should still take precautions.  However, since most file
systems embody the notion of a file name and a file type, this convention will
allow these items to be expressed in a way that an unlike system can
understand.  The particular notation is chosen simply because it is the most
common.

The recipient must worry about the length of the name and type fields of the
file name.  If either is too long, they must be truncated.  If the result
(whether truncated or not) is the same as the name of a file that already
exists in the same area, the recipient should have the ability to take some
special action to avoid writing over the original file.

KERMIT implementations that convert file specifications to normal form by
default should have an option to override this feature.  This would be most
useful when transferring files between like systems, perhaps used in
conjunction with "image mode" file transfer.  This could allow, for instance,
one UNIX system to send an entire directory tree to another UNIX system.

@Section<Robustness>

A major feature of the KERMIT protocol is the ability to transfer multiple
files.  Whether a particular KERMIT program can actually send multiple files
depends on the capabilities of the program and the host operating system
(any KERMIT program can receive multiple files).

If a KERMIT program can send multiple files, it should make every attempt to
send the entire group specified.  If it fails to send a particular file, it
should terminate the entire batch, but should go on the the next one, and
proceed until an attempt has been made to send each file in the group.

Operating in this robust manner, however, gives rise to a problem: the user
must be notified of a failure to send any particular file.  Unfortunately, it
is not sufficient to print a message to the screen; users who initiate long
batch transfers may have gone out to lunch and such messages may be long gone
by the time they return.  Furthermore, there is no way for a remote sender to
inform the local receiver of such an error without killing the whole batch
transfer, since Error (E) packets always cause the transaction to terminate.

One solution might be to invent a non-fatal warning or general purpose display
message packet, but this would still not help the user who was not there to see
it.  A better solution would be to have the sender optionally keep a log of the
@Index<Transaction Log>
transaction, giving the name of each file for which an attempt was made, and
stating whether the attempt was successful, and if not, the reason.

@Section<Flow Control>

@label<-flow>
@Index<XON/XOFF>@Index<Flow Control>
On full duplex connections,
XON/XOFF flow control can generally be used in conjunction with KERMIT file
transfer with no ill effects.  This is because XOFFs are sent in the
opposite direction of packet flow, so they will not interfere with the packets
themselves.  XON/XOFF, therefore, need not be implemented by the KERMIT
program, but can done by the host system.  If the host system provides this
capability, it should be used -- if both sides can respond XON/XOFF signals,
then buffer overruns and the resulting costly packet retransmissions can be
avoided.

Flow control via modem signals can also be used when available.

@Chapter<Basic KERMIT Protocol State Table>

The KERMIT protocol can be described as a set of states and transitions, and
rules for what to do when changing from one state to another.  State changes
occur based on the type of packets that are sent or received, or
errors that may occur.  Packets always go back and forth; the sender of a file
always sends data packets of some kind (init, header, data) and the receiver
always returns ACK or NAK packets.  Timeouts@index<Timeout> and error
conditions have been omitted from the following state table for simplicity, but
the action is as described above.

 Upon entering a given state, a certain kind of packet is either being sent or
is expected to arrive -- this is shown on top of the description of that state.
As a result of the action, various events may take place; these are shown in
the EVENT column.  For each event, an appropriate ACTION is taken, and the
protocol enters a NEW STATE.
@newpage()
@MajorHeading(KERMIT Protocol State Table)

@begin<example>

@u<STATE   EVENT           ACTION                      >@ux<NEW ST@^ATE>

@i(@ @ @ @ @ -- SEND STATES --)

@!@i(Send Send-Init Packet)@/@&_@\
S       Get NAK,bad ACK (None)                          S
        Get good ACK    Set remote's params, open file  SF
        (Other)         (None)                          A
        
@i(Send File-Header Packet)@/@&_@\
SF      Get NAK,bad ACK (None)                          SF
        Get good ACK    Get bufferful of file data      SD
        (Other)         (None)                          A

@i(Send File-Data Packet)@/@&_@\
SD      Get NAK,bad ACK (None)                          SD
        Get good ACK    Get bufferful of file data      SD
        (End of file)   (None)                          SZ
        (Other)         (None)                          A

@i(Send EOF Packet)@/@&_@\
SZ      Get NAK,bad ACK (None)                          SZ
        Get good ACK    Get next file to send           SF
        (No more files) (None)                          SB
        (Other)         (None)                          A

@i<Send Break (EOT) Packet>@/@&_@\
SB      Get NAK,bad ACK (None)                          SB
        Get good ACK    (None)                          C
        (Other)         (None)                          A


@i<@ @ @ @ @ -- RECEIVE STATES -->

@i(Wait for Send-Init Packet)@/@&_@\
R       Get Send-Init   ACK w/local params              RF
        (Other)         (None)                          A

@i(Wait for File-Header Packet)@/@&_@\
RF      Get Send-Init   ACK w/local params
                         (previous ACK was lost)        RF
        Get Send-EOF    ACK (prev ACK lost)             RF
        Get Break       ACK                             C
        Get File-Header Open file, ACK                  RD
        (Other)         (None)                          A

@i(Wait for File-Data Packet)@/@&_@\
RD      Get previous    
         packet(D,F)    ACK it again                    RD
        Get EOF         ACK it, close the file          RF
        Get good data   Write to file, ACK              RD
        (Other)         (None)                          A


@i<@ @ @ @ @ -- STATES COMMON TO SENDING AND RECEIVING -->

C       (Send Complete)                                 start
A       ("Abort")                                       start
@end<example>

@Chapter<Optional Features>

The foregoing sections have discussed basic, required operations for any KERMIT
implementation.   The following sections discuss optional features.

@Section<8th-Bit and Repeat Count Prefixing>

Prefix quoting of control characters is mandatory.  In addition, prefixing may
also be used for 8-bit quantities or repeat counts, when both KERMIT programs
agree to do so.  8th-bit prefixing can allow 8-bit binary data pass through
7-bit physical links.  Repeat count prefixing can improve the throughput of
certain kinds of files dramatically; binary files (particularly executable
programs) and structured text (highly indented or columnar text) tend to be the
major beneficiaries.

When more than one type of prefixing is in effect, a single data character can
be preceded by more than one prefix character.  A receiver will never do any
kind of prefixing, since prefixing can only occur in the data field, and the
receiver only responds with ACKs and NAKs with empty or special data fields.
Repeat count processing can only be requested by the sender, and will only be
used by the sender if the receiver agrees.  8th-bit prefixing is a special case
because it is normally not desirable to use it, since it increases both
processing and transmission overhead.  However, since it is the only
straightforward mechanism for binary file transfer available to those systems
that usurp the parity bit, a receiver must be able to request the sender to do
8th-bit quoting, since most senders will not normally do it by default.

The repeat prefix is followed immediately by a single-@|character repeat count,
encoded printably via @q<char()>, followed by the character itself (perhaps
prefixed by control or 8th bit quotes, as explained below).  The repeat count
may express values from 0 to 94.  If a character appears more than 94
times in a row, it must be "cut off" at 94, emitted with all appropriate
prefixes, and "restarted".
The following table should clarify Kermit's quoting mechanism (the final line
shows how a sequence of 120 consecutive NULs would be encoded):
@begin<example>
             Quoted             With
@u<Character    Representation>     @ux<Repeat Count for 6>
   A            A                ~(A   ["(" is ASCII 40 - 32 = 6]
   ^A           #A               ~(#A
   'A           &A               ~(&A
   '^A          &#A              ~(&#A
   #            ##               ~(##
   '#           &##              ~(&##
   &            #&               ~(#&
   '&           &#&              ~(&#&
   ~            #~               ~(#~
   '~           &#~              ~(&#~
   NUL          #@@               ~~#@@~:#@@ [120 NULs]
@end<example>
@q<A> represents any printable character, @q<^A> represents any control
character, @q<'x> represents any character with the 8th bit set.  The @q<#>
character is used for control-@|character quoting, and the @q<&> character for
8-bit quoting.  The repeat count must always precede any other prefix
character.  The repeat count is taken literally (after transformation by
@q<unchar()>; for instance "@q<#>" and "@q<&>" immediately following a "@q<~>"
denote 
repeat counts, not control characters or 8-bit characters.  The control quote
character "@q<#>" is most closely bound to the data character, then the
8-bit prefix, then the repeat count; in other words, the order should
be: repeat prefix and count, 8-bit quote, control quote, and the data character
itself.  To illustrate, note that @q<&#A> is @i<not> equivalent to @q<#&A>.
And finally, remember:
@Begin(Itemize)
@i<Prefixed sequences must not be broken across packets.>

@i<Control, 8th-bit, and repeat count prefixes must be distinct.>
@End(Itemize)

@Section<Server Operation>

A KERMIT server is a KERMIT program running remotely with no "user interface".
All commands to the server arrive in packets from the local KERMIT.  SERVER
operation is much more convenient than basic operation, since the user need
never again interact directly with the remote KERMIT program after starting it
up in server mode, and therefore need not issue complementary SEND and RECEIVE
commands on the two sides to get a file transfer started; rather, a single
command (SEND or GET) to the local KERMIT suffices.  KERMIT servers can also
provide services beyond file transfer.

Between transactions, a Kermit server waits for packets containing server
commands.  The packet sequence number is always set back to 0 after a
transaction.  A Kermit server in command wait should be looking for packet 0,
and command packets sent to servers should also be packet 0.
Certain server commands will result in the exchange of multiple packets.  Those
operations proceed exactly like file transfer.

Server operation must be implemented in two places: in the server itself, and
in any KERMIT program that will be communicating with a server.  The server
must have code to read the server commands from packets and respond to them.
The user KERMIT must have code to parse commands to send requests to
servers, to form the server command packets, and to handle the responses to
those server commands.

Server commands are listed below.  Not all of them have been implemented, and
some may never be, but their use should be reserved.  Although server-@|mode
operation is optional, certain commands should be implemented in every server.
These include Send-Init (S), Receive-Init (R), and the Generic Logout (GL)
and/or Finish (GF) commands.  If the server receives a command it does not
understand, or cannot execute, it should respond with an Error (E) packet
containing a message like "Unimplemented Server Command" and both sides should
set the packet sequence number back to 0, but the server should remain in
server command wait.  Only a GL or GF command should terminate server
operation.

Server commands are as follows:
 @begin<description,spread 0, leftmargin +4, indent -4>
S@\Send Initiate (exchange parameters, server waits for a file).

R@\Receive Initiate (ask the server to send the specified files).

I@\Initialize (exchange parameters).

C@\Host Command.  The data field contains a string to be executed as a command
by the host system command processor.

G@\Generic Kermit Command.  Single character in data field (possibly followed
by operands, shown in {braces}, optional fields in [brackets]) specifies the
command:
 @begin<description,spread 0, leftmargin +4, indent -4>
I@\Login [{~user[~password[~account]]}]

C@\CWD, Change Working Directory [{~directory[~password]}]

L@\Logout, Bye

F@\Finish (Shut down the server, but don't logout).

D@\Directory [{~filespec}]

U@\Disk Usage Query [{~area}]

E@\Erase (delete) {~filespec}

T@\Type {~filespec}

R@\Rename {~oldname~newname}

K@\Copy {~source~destination}

W@\Who's logged in? (Finger) [{~user ID or network host[~options]}]

M@\Send a short Message {~destination~text}

H@\Help [{~topic}]

Q@\Server Status Query
@end<description>
Tilde as used above ("~") is a length field, encoded using @q<char()>,
for the operand that follows it.@foot<This is an incompatible change from
previous editions of this manual, in which the tilde was inserted literally.>

@blankspace(1)
@begin[multiple]
X@\Text header.  Allows transfer of text to the user's screen in
response to a generic or host command.  This works just like file transfer
except that the destination "device" is the screen rather than a file.

KERMIT does not provide a mechanism for suspending and continuing a
transaction.  This means that text sent to the user's screen should not be
frozen for long periods (i.e.@ not longer than the timeout period times the
retry threshold).
@end[multiple]
@end<description>
 Between transactions, when the server has no tasks pending, it may send out
periodic NAKs (always with type 1 checksums) to prevent a deadlock in case a
command was sent to it but was lost.  These NAKs can pile up in the local
"user" Kermit's input buffer (if it has one), so the user Kermit should be
prepared to clear its input buffer before sending a command to a server.

@SubSection<The R Command>

The R packet tells the server to send the files specified by the name in the
data field of the R packet.  Since we can't assume that the two Kermits are
running on like systems, the local (user) Kermit must parse the file
specification as a character string and let the server to check it.  If the
server likes the filespec, it sends a Send-Init (S) packet -- @i<not an
acknowledgement!> -- to the user, and then completes the file-@|sending
transaction, as described above.

If the server cannot send the file, it should respond with an error packet
containing a reason, like "File not found" or "Read access required".

@SubSection<Short and Long Replies>

 Any request made of a server may be answered in either of two ways, and any
User Kermit that makes such a request should be prepared for either kind of
reply:
@Begin(itemize)
@i<A short reply>.  This consists of a single ACK packet, which may contain
text in its data field.  For instance, the user might send a disk space query
to the server, and the server might ACK the request with a short character
string in the data field, such as "12K bytes free".  The user KERMIT should
display this text on the screen.

@i<A long reply>.  This proceeds exactly like a file transfer (and in some
cases it may be a file transfer).  It begins with either a File-Header (F)
packet (optionally followed by one or more Attributes packets@foot<See section
@ref<-attributes> for a description of the Attributes packet.>), or a
Text-Header (X) packet.  Then comes an arbitrary number of Data (D) packets,
then an End-Of-File (Z) packet, and finally a Break-Transmission (B) packet.
@End(itemize)

@SubSection<Additional Server Commands>

The following server commands request the server to perform
tasks other than sending or receiving files.  Almost any of these can have
either short or long replies.  For instance, the Generic Erase (GE) command may
elicit a simple ACK, or a stream of packets containing the names of all the
files it erased (or didn't erase).  These commands are now described in more
detail; arguments are as provided in commands typed to the user KERMIT; no
transformations to any kind of normal or canonic form are done -- filenames and
other operands are in the syntax of the server's host system.

@begin<description, leftmargin +4, indent -4>
I@\Login.  For use when a KERMIT server is kept perpetually running on a
dedicated line.  This lets a new user obtain an identity on the server's host
system.  If the data field is empty, this removes the user's identity, so that
the next user does not get access to it.  A dedicated server such as this
should probably disallow the L and F generic commands, listed below.

L@\Logout, Bye.  This shuts down the server entirely, causing the server itself
to log out its own job.  This is for use when the server has been started up
manually by the user, who then wishes to shut it down remotely.

F@\Finish.  This is to allow the user to shut down the server, putting its
terminal back into normal (as opposed to binary or raw) mode, and putting the
server's job back at system command level, still logged in, so that the user
can connect back to the job.

C@\CWD.  This sets the default directory or area for file transfer on the
server's host.  With no operands, this command sets the default area to be the
user's own default area.

D@\Directory.  Send a directory listing to the user.  The user program can
display it on the terminal or store it in a file, as it chooses.  The directory
listing should contain file sizes and creation dates as well as file names, if
possible.  A wildcard or other file-@|group designator may be specified to ask
the server list only those files that match.  If no operand is given, all files
in the current area will be shown.

U@\Disk Usage Query.  The server responds with the amount of space used and the
amount left free to use, in K bytes (or other units, which should be
specified).

E@\Erase (delete).  Delete the specified file or file group.

T@\Type.  Send the specified file or file group, indicating (by starting with
an X packet rather than an F packet, or else by using the Type attribute) that
the file is to be displayed on the screen, rather than stored.

R@\Rename.  Change the name of the file as indicated.  The string indicating
the new name may contain other attributes, such as protection code, permitted
by the host.

K@\Copy.  Produce a new copy of the file or file group, as indicated, leaving
the source file(s) unmodified.

W@\Who's logged in? (Finger).  With no arguments, list all the users who are
logged in on the server's host system.  If an argument is specified, provide
more detailed information on the specified user or network host.

M@\Short Message.  Send the given short (single-packet) message to the
indicated user's screen.
@end<description>

@SubSection<Host Commands>

Host commands are conceptually simple, but may be hard to implement on some
systems.  The C packet contains a text string in its data field which is simply
fed to the server's host system command processor; any output from the
processor is sent back to the user in KERMIT packets, as either a short or long
reply.

Implementation of this facility under UNIX, with its forking process structure
and i/o redirection via pipes, is quite natural.  On other systems, it could be
virtually impossible.

@SubSection<Exchanging Parameters Before Server Commands>

In basic KERMIT, the Send-Init exchange is always sufficient to configure the
two sides to each other.  During server operation, on the other hand, some
transactions may not begin with a Send-Init packet.  For instance, when the
user sends an R packet to ask the server to send a file, the server chooses
what block check option to use.  Or if the user requests a directory listing,
the server does not know what packet length to use.

The solution to this problem is the "I" (Init-Info) packet.  It is exactly like
a Send-Init packet, and the ACK works the same way too.  However, receipt of an
I packet does not cause transition to file-send state.  The I-packet exchange
simply allows the two sides to set their parameters, in preparation for the
next transaction.

Servers should be able to receive and ACK "I" packets when in server command
wait.  User KERMITs need not send "I" packets, however; in that case, the
server will assume all the defaults for the user listed on
page @pageref<-defaults>.

@section<Alternate Block Check Types>

There are two optional kinds of block checks:
@begin<description,leftmargin +4,indent -4>
@ux<Type 2>@\
A two-@|character checksum based on the low order 12 bits of the arithmetic sum
of the characters in the packet (from the LEN field through the last data
character, inclusive) as follows:
@begin<example>
         1              2
--------+--------------+-------------+
...data | char(b6-b11) | char(b0-b5) |
--------+--------------+-------------+
@End(Example)
For instance, if the 16-bit result is 154321 (octal), then the 2 character
block check would be "@q<C1>".

@ux<Type 3>@\ Three-@|character 16-bit CRC-CCITT.  The CRC calculation treats the
data it operates upon as a string of bits with the low order bit of the first
character first and the high order bit of the last character last.  The initial
value of the CRC is taken as 0; the 16-bit CRC is the remainder after dividing
the data bit string by the polynomial @i<X>@+(16)+@i<X>@+(12)+@i<X>@+(5)+1
(this calculation can actually be done a character at a time, using a simple
table lookup algorithm).  The result is represented as three printable
characters at the end of the packet, as follows:
@begin<example>
         1               2              3
--------+---------------+--------------+-------------+
...data | char(b12-b15) | char(b6-b11) | char(b0-b5) |
--------+---------------+--------------+-------------+
@End(Example)
For instance, if the 16-bit result is 154321 (octal), then the 3 character
block check would be "@q<-C1>".
@End(Description) 
The single-character checksum has proven quite adequate in practice.  The other
options can be used only if both sides agree to do so via Init packet (S or I)
exchange.  The 2 and 3 character block checks should only be used
under conditions of severe line noise and packet corruption.

Since type 2 and 3 block checks are optional, not all KERMITs can be expected
to understand them.  Therefore, during initial connection, communication must
begin using the type 1 block check.  If type 2 or 3 block checks are agreed to
during the "I" or "S" packet exchange, the switch will occur @i<only after> the
Send-Init has been sent and ACK'd with a type 1 block check.  This means that
the first packet with a type 2 or 3 block check must always be an "F" or "X"
packet.  Upon completion of a transaction, both sides must switch back to type
1. The transaction is over @i<after> a "B" or "E" packet has been sent and
ACK'd, or after any error that terminates the transaction prematurely or
abnormally.

A server that has completed a transaction and is awaiting a new command may
send out periodic NAKs for that command (packet 0).  Those NAKs must have type
1 block checks.

@section<Interrupting a File Transfer>

This section describes an optional feature of the KERMIT protocol to allow
graceful interruption of file transfer.  This feature is unrelated to server
operation.

To interrupt sending a file, send an EOF ("Z") packet in place of the next data
packet, including a "D" (for Discard) in the data field.  The recipient ACKs
the Z packet normally, but does not retain the file.  This does not interfere
with older Kermits on the receiving end; they will not inspect the data field
and will close the file normally.  The mechanism can be triggered by typing an
interrupt character at the console of the sending KERMIT program.  If a
(wildcard) file group is being sent, it is possible to skip to the next file or
to terminate the entire batch; the protocol is the same in either case, but the
desired action could be selected by different interrupt characters, e.g. CTRL-X
to skip the current file, CTRL-Z to skip the rest of the batch.

To interrupt receiving a file, put an "X" in the data field of an ACK for a
data packet.  To interrupt receiving an entire file group, use a "Z".  The user
could trigger this mechanism by typing an interrupt character by typing, say,
CTRL-X and CTRL-Z, respectively, at the receiving KERMIT's console.  A sender
that was aware of the new feature, upon finding one of these codes, would act
as described above, i.e. send a "Z" packet with a "D" code; a sender that did
not implement this feature would simply ignore the codes and continue sending.
In this case, and if the user wanted the whole batch to be cancelled (or only
one file was being sent), the receiving KERMIT program, after determining that
the sender had ignored the "X" or "Z" code, could send an Error (E) packet to
stop the transfer.

The sender may also choose to send a Z packet containing the D code when the it
detects that the file it is sending cannot be sent correctly and completely --
for instance, after sending some packets correctly, it gets an i/o error
reading the file.  Or, it notices that the "8th bit" of a file byte is set when
the file is being sent as a text file and no provision has been made for
transmitting the 8th bit.

@Section<Transmitting File Attributes>
@label<-attributes>

The optional Attributes (A) packet provides a mechanism for the sender of a
file to provide additional information about it.  This packet can be sent if
the receiver has indicated its ability to process it by setting the Attributes
bit in the capability mask.  If both sides set this bit in the Kermit
Capability Word, then the sender, after sending the filename in the "F" packet
and receiving an acknowledgement, may (but does not have to) send an "A" packet
to provide file attribute information.

Setting the Attributes bit in the capability mask does @i<not> indicate support
for any particular attributes, only that the receiver is prepared to accept the
"A" packet.

The attributes are given in the data field of the "A" packet.  The data
field consists of 0 or more subfields, which may occur in any order.  Each
subfield is of the following form:
@Begin(Example) 
+-----------+--------------+------+
| ATTRIBUTE | char(LENGTH) | DATA |
+-----------+--------------+------+
@End(Example)
where
@Begin(Description, leftmargin +10, indent -10) 
ATTRIBUTE@\is a single printable character other than space,

LENGTH@\is the length of the data characters (0 to 94), with 32
added to produce a single printable character, and

DATA@\is @i<length> characters worth of data, all printable characters.
@End(Description) 
No quoting or prefixing is done on any of this data.

More than one attribute packet may be sent.  The only requirement is that all
the A packets for a file must immediately follow its File header (or X) packet,
and precede the first Data packet.

There may be 93 different attributes, one for each of the 93 printable ASCII
characters other than space.  These are assigned in ASCII order.
@Begin(Description, leftmargin +10, indent -10) 
@q<!> (ASCII 33)@\Length.  The data field gives the length in K (1024) bytes,
as a printable decimal number, e.g. "!#109".  This will allow
the receiver to determine in advance whether there is sufficient
room for the file, and/or how long the transfer will take.

@q<"> (ASCII 34)@\Type.  The data field can contain some indicator of the
nature of the file.  Note that only sequential files can be supported
by the KERMIT protocol.  Operands are enclosed in {braces}, optional items in
[brackets].
@Begin(Description,leftmargin +8, indent -8)
A[{xx}]@\ASCII text, containing no 8-bit quantities, logical records (lines)
delimited by the (quoted) control character sequence {xx},
represented here by its printable counterpart (MJ = CRLF,
J = LF, etc).  For instance AMJ means that the appearance
of @q<#M#J> (the normal prefixed CRLF sequence) in a file data
packet indicates the end of a record.  If {xx} is omitted, MJ will be assumed.

B[{xx}]@\Binary.  {xx} indicates in what manner the file is binary:
@Begin(Description,leftmargin +4, indent -4)
8@\(default) The file is a sequence of 8-bit bytes, which must be
saved as is.  The 8th bit may be sent "bare", or prefixed
according to the Send-Init negotiation about 8th-bit
prefixing.

36@\The file is a PDP-10 format binary file, in which five
7-bit bytes are fit into one 36-bit word, with the final
bit of each word being represented as the "parity bit" of
every 5th character (perhaps prefixed).
@End(Description) 

D{x}@\Variable length undelimited records.  Each logical record begins with an
{x}-character ASCII length field (similar to ANSI tape format "D").

F{xxxx}@\Fixed-length undelimited records.  Each logical record is
{xxxx} bytes long.

I[{x}]@\Image.  The file is being sent exactly as it is represented on the
system of origin.  For use between like systems.  There are {x} usable bits per
character, before prefixing.  For instance, to send binary data from a system
with 9-bit bytes, it might be convenient to send three 6-bit characters for
every two 9-bit bytes.  Default {x} is 8.

@End(Description) 

@q<#> (ASCII 35)@\Creation Date, expressed as "@q<[yy]yymmdd[ hh:mm[:ss]]>"
(ISO standard format), e.g. @q<831009@ 23:59>.  The time is optional; if given,
it should be in 24-hour format, and the seconds may be omitted, and a single
space should separate the time from the date.

@q<$> (ASCII 36)@\Creator's ID, expressed as a character string of the given
length. 

@q<%> (ASCII 37)@\Account to charge the file to, character string.

@q<&> (ASCII 38)@\Area in which to store the file, character string.

@q<'> (ASCII 39)@\Password for above, character string.

@q<(> (ASCII 40)@\Block Size.  The file is to be stored with the given block
size. This may be useful when sending files to or from IBM mainframes,
VAX/VMS, or other systems where files may have this attribute.

@q<)> (ASCII 41)@\Access:
@Begin(Description,leftmargin +4, indent -4,spread 0)
N@\New, the normal case -- create a new file of the given name.

S@\Supersede (overwrite) any file of the same name.

A@\Append to file of the given name.
@End(Description)

@q<*> (ASCII 42)@\Encoding:
@Begin(Description,leftmargin +4, indent -4,spread 0)
A@\ASCII, normal ASCII encoding with any necessary prefixing, etc.
  
H@\Hexidecimal "nibble" encoding.

X@\Encrypted.

E@\EBCDIC (sent as if it were a binary file).

S@\Sixbit Character set (equivalent to I6).

R@\Radix50
@End(Description) 

@q<#> (ASCII 43)@\Disposition (operands are specified in the syntax of the
receiver's host system):
@Begin(Description) 
M{user(s)}@\Send the file as Mail to the specified user(s).

O{destination}@\Send the file as a lOng terminal message to the specified
destination (terminal, job, or user).

S[{options}]@\Submit the file as a batch job, with any specified options.

P[{options}]@\Print the file on a system printer, with any specified options,
which may specify a particular printer, forms, etc.

T@\Type the file on the screen.

L[{aaa}]@\Load the file into memory at the given address, if any.

X[{aaa}]@\Load the file into memory at the given address and eXecute it.
@End(Description) 

, (ASCII 44)@\Protection.  Protection code for the file, in the syntax of the
receiver's host file system.  With no operand, store according to the system's
default protection for the destination area.

- (ASCII 45)@\Protection.  Protection code for the file with respect to the
"public" or "world", expressed in generically in a 6-bit quantity (made
printable by @q<char()>), in which the bits have the following meaning:
@Begin(Description,leftmargin +4,indent -4,spread 0)
b0:@\Read Access

b1:@\Write Access

b2:@\Execute Access

b3:@\Append Access

b4:@\Delete Access

b5:@\Directory Listing
@End(Description)
A one in the bit position means allow the corresponding type of access, a zero
means prohibit it.
@End(Description)
Others can be imagined, and can be added later if needed.  However, two
important points should be noted:
@Begin(Itemize)
The receiver may have absolutely no way of honoring, or even recording, a given
attribute.  For instance, CP/M-80 has no slot for creation date or creator's ID
in its FCB; the DEC-20 has no concept of block size, etc.

The sender may have no way of determining the correct values of any of the
attributes.  This is particularly true when sending files of foreign origin.
@End(Itemize)

The "A" packet mechanism only provides a way to send certain information about
a file to the receiver, with no provision or guarantee about what the receiver
may do with it.  That information may be obtained directly from the file's
directory entry (FCB, FDB, @q<...>), or specified via user command.

The ACK to the "A" packet may in turn have information in its data field.
However, no complicated negotiations about file attributes may take place, so
the net result is that the receiver may either refuse the file or accept it.
The receiver may reply to the "A" packet with any of the following codes in the
data field of the ACK packet:
@Begin(Description,leftmargin +8, indent -8)
<null>@\(empty data field) I accept the file, go ahead and send it.

N[{xxx}]@\I refuse the file as specified, don't send it; {xxx} is a string of
zero or more of the attribute characters listed above, to specify what
attributes I object to (e.g. "@q<!>" means it's too long, "@q<&>" means I don't
have write access to the specified area, etc).

Y[{xxx}]@\I agree to receive the file, but I cannot honor attributes {xxx}, so
I will store the file according to my own defaults.

Y@\(degenerate case of Y{xxx}, equivalent to <null>, above)
@End(Description)

How the receiver actually replies is an implementation decision.  A NAK in
response to the "A" packet means, of course, that the receiver did not receive
the "A" correctly, not that it refuses to receive the file.

@Chapter<KERMIT Commands>

The following list of KERMIT commands and terms is suggested.  It is not
intended to recommend a particular style of command parsing, only to promote a
consistent vocabulary, both in documentation and in choosing the names for
commands.

@Section<Basic Commands>

@Begin(Description,leftmargin +8,indent -8)
SEND@\This verb tells a Kermit program to send one or more files from its own
  file structure.

RECEIVE@\This verb should tell a Kermit program to expect one or more files to
  arrive.

GET@\This verb should tell a user Kermit to send one or more files.  Some
  Kermit implementations have separate RECEIVE and GET commands; others use
  RECEIVE for both purposes, which creates confusion.
@End(Description)
Since it can be useful, even necessary, to specify different names for source
and destination files, these commands should take operands as follows (optional
operands in [brackets]):
@Begin(Description,leftmargin +8,indent -8)
SEND local-source-filespec [remote-destination-filespec]@\
If the destination file specification is included, this will go in the file
header packet, instead of the file's local name.

RECEIVE [local-destination-filespec]@\
If the destination filespec is given, the incoming file will be stored under
that name, rather than the one in the file header pakcet.

GET remote-source-filespec [local-destination-filespec]@\
If the destination filespec is given, the incoming file will be stored under
that name, rather than the one in the file header packet.
@End(Description)
If a file group is being sent or received, alternate names should @i<not> be
used.

@Section<Program Management Commands>

@Begin(Description,leftmargin +8,indent -8)
EXIT@\Leave the KERMIT program, doing whatever cleaning up must be done --
deassigning of devices, closing of files, etc.

QUIT@\Leave the KERMIT program without cleaning up, in such a manner as to
allow further manipulation of the files and devices.

PUSH@\Preserve the current KERMIT environment and enter the system command
processor.

TAKE@\Read and execute KERMIT program commands from a local file.
@End(Description)

@Section<Terminal Emulation Commands>
@label<-emulation>

@Begin(Description,leftmargin +8,indent -8)
CONNECT@\This verb, valid only for a local Kermit, means to go into terminal
emulation mode; present the illusion of being directly connected as a terminal
to the remote system.  Provide an "escape character" to allow the user to "get
back" to the local system.  The escape character, when typed, should take a
single-@|character argument; the following are suggested:
@Begin(Description,leftmargin +8,indent -4,spread 0)
0@\Transmit a NUL

B@\Transmit a BREAK

C@\Close the connection, return to local KERMIT command level

P@\Push to system command processor

Q@\Quit logging (if logging is being done)

R@\Resume logging

S@\Show status of connection

?@\Show the available arguments to the escape character

@i<(a second copy of the escape character)>@\Transmit the escape
character itself
@End(Description)
Lower case equivalents should be accepted.  If any invalid argument is typed,
issue a beep.
@End(Description)
Also see the SET command.

@Section<Special User-Mode Commands>

These commands are used only by Users of Servers.

@Begin(Description,leftmargin +8,indent -8)
BYE@\This command sends a message to the remote server to log itself out, and
  upon successful completion, the local Kermit program to terminate.

FINISH@\This command causes the remote server to shut itself down gracefully
without logging out its job.
@End(Description) 

@Section<Commands Whose Object Should Be Specified>

Some Kermit implementations include various local file management services and
commands to invoke them.  For instance, an implementation might have commands
to let you get directory listings, delete files, switch disks, and inquire
about free disk space without having to exit and restart the program.  In
addition, remote servers may also provide such services.  A user Kermit must
be able to distinguish between commands aimed at its own system and those
aimed at the remote one.  When any confusion is possible, such a command may be
prefixed by one of the following "object prefixes":

@Begin(Description,leftmargin +8,indent -8)
REMOTE@\Ask the remote Kermit server to provide this service.

LOCAL@\Perform the service locally.
@End(Description) 

If the "object prefix" is omitted, the command should be executed locally.
The services include:

@Begin(Description,leftmargin +8,indent -8)
LOGIN@\This should be used in its timesharing sense, to create an identity
("job", "session", "access", "account") on the system.

CWD@\Change Working Directory.  This is ugly, but more natural verbs like
CONNECT and ATTACH are too imprecise.  CWD is the ARPAnet file transfer
standard command to invoke this function.

DIRECTORY@\Provide a list of the names, and possibly other attributes, of the
files in the current working directory (or the specified directory).

SPACE@\Tell how much space is used and available for storing files in the
current working directory (or the specified directory).

DELETE@\Delete the specified files.

ERASE@\This could be a synomym for DELETE, since its meaning is clear.
@Begin(Quotation)
(It doesn't seem wise to include UNDELETE or UNERASE in the standard
list; most systems don't support such a function, and users' expectations
should not be toyed with...)
@End(Quotation)

COPY@\Make a new copy of the specified file with the specified name.

RENAME@\Change the name of the specified file.

TYPE@\Display the contents of the specified file(s) at the terminal.

SUBMIT@\Submit the specified file(s) for background (batch) processing.

PRINT@\Print the specified file(s) on a printer.

MOUNT@\Request a mount of the specified tape, disk, or other removable storage
medium.

WHO@\Show who is logged in (e.g. to a timesharing system), or give
information about a specified user or network host.

MAIL@\Send electronic mail to the specified user(s).

MESSAGE@\Send a terminal message (on a network or timesharing system).

HELP@\Give brief information about how to use KERMIT.

SET@\Set various parameters relating to debugging, transmission, file mode,
and so forth.

SHOW@\Display settings of SET parameters, capabilities in force, etc.

STATISTICS@\Give information about the performance of the most recent file
transfer -- elapsed time, effective baud rate, various counts, etc.

HOST@\Pass the given command string to the specified (i.e. remote or local)
host for execution in its own command language.
@End(Description) 

@section<The SET Command>

A SET command should be provided to allow the user to tailor a connection to
the peculiarities of the communication path, the local or remote file system,
etc.  Here are some parameters that should be SET-able:
@Begin(Description,leftmargin +8,indent -8)
FILE attributes@\
Almost any of the attributes listed above in the Attributes section
(@ref<-attributes>).  The most common need is to tell the KERMIT program
whether an incoming or outbound file is text or binary.

DEBUGGING@\Display or log the packet traffic, packet numbers, and/or program
states.  Useful for debugging new versions of KERMIT, novel combinations of
KERMIT programs, etc.

DELAY@\How many seconds a remote (non-server) KERMIT should wait before sending
the Send-Init packet, to give the user time to escape back to the local KERMIT
and type a RECEIVE command.

DUPLEX@\For terminal emulation, specify FULL or HALF duplex echoing.

END-OF-LINE@\Specify any line terminator that must be used after a packet.

ESCAPE@\Specify the escape character for terminal emulation.

HANDSHAKE@\Specify any line-@|access negotiation that must be used or simulated
during file transfer.  For instance, a half duplex system will often need to
"turn the line around" after sending a packet, in order to give you permission
to reply.  A common handshake is XOFF; the current user of the line transmits
an XOFF when done transmitting data.

LINE@\Specify the line or device designator for the connection.  This is for
use in a KERMIT program that can run in either remote or local mode; the
default line is the controlling terminal (for remote operation).  If an
external device is used, local operation is presumed.

LOG@\Specify a local file in which to keep a log of the transaction.  There may
be logs for debugging purposes (packet traffic, state transitions, etc) and for
auditing purposes (to record the name and disposition of each file
transferred).

MARKER@\Change the start-of-packet marker from the default of SOH (CTRL-A) to
some other control character, in case one or both systems has problems using
CTRL-A for this purpose.

PACKET-LENGTH@\The maximum length for a packet.  This should normally be no
less than 30 or 40, and can never be greater than 96.  Short packets can be an
advantage on noisy lines; they reduce the probabily of a particular packet
being corrupted, as well as the retransmission overhead when corruption does
occur.

PADDING@\The number of padding characters that should be sent before each
packet, and what the padding character should be.  Rarely necessary.

PARITY@\Specify the parity (ODD, EVEN, MARK, SPACE, NONE) of the physical
connection.  If other than none, the "8th bit" cannot be used to transmit data
and must not be used by either side in block check computation.

PAUSE@\How many seconds to pause after receiving a packet before sending the
next packet.  Normally 0, but when a system communication processor or front
end has trouble keeping up with the traffic, a short pause between packets may
allow it to recover its wits; hopefully, something under a second will suffice.

PREFIX@\Change the default prefix for control characters, 8-bit characters, or
repeated quantities.

EIGHT-BIT-PREFIXING@\Specify that "8th bit" prefixing must be done; normally it
will not be done.

REPEAT-COUNT-PROCESSING@\Change the default for repeat count processing.
Normally, it will be done if both KERMIT programs agree to do it.

RETRY@\The maximum number of times to attempt to send or receive a packet
before giving up.  The normal number is about 5, but the user should be able to
adjust it according to the condition of the line, the load on the systems, etc.

TIMEOUT@\Specify the length of the timer to set when waiting for a packet to
arrive.
@End(Description)

@Section<Macros>

In addition to the individual set commands, a "macro" facility is recommended
to allow users to combine the characteristics of specific systems into a single
SET option.  For example:
@begin[example]
DEFINE IBM = PARITY ODD, DUPLEX HALF, HANDSHAKE XOFF
DEFINE UNIX = PARITY NONE, DUPLEX FULL
DEFINE TELENET = PARITY MARK
@end[example]
This could be done by providing a fancy runtime parser for commands like this
(which could be automatically TAKEn from the user's KERMIT initialization file
upon program startup), or simply hardwired into the SET command table.

With these definitions in place, the user would simply type "SET IBM", "SET
UNIX", and so forth, to set up the program to communication to the remote
system.

@chapter<Terminal Emulation>

The local system must be able to act as a terminal so that the user can connect
to the remote system, log in, and start up the remote KERMIT.

Terminal emulation should be provided by any KERMIT program that runs locally,
so that the user need not exit and restart the local KERMIT program in order to
switch between terminal and protocol operation.  On smaller systems, this is
particularly important for various reasons -- restarting the program and typing
in all the necessary SET commands is too inconvenient and time-@|consuming;
in some micros, switching in and out of terminal emulation may cause carrier to
drop, etc.

Only bare-bones terminal emulation need be supplied by KERMIT; there is no need
to emulate any particular kind of "smart" terminal.  Simple "dumb" terminal
emulation is sufficient to do the job.  Emulation of fancier terminals is nice
to have, however, to take advantage of the remote system's editing and display
capabilities.  In some cases, microcomputer firmware will take care of this.
To build emulation for a particular type of terminal into the program, you must
interpret and act upon escape sequences as they arrive at the port.

No error checking is done during terminal emulation.  It is "outside the
protocol"; characters go back and forth "bare".  Terminal emulation through
KERMIT is no better than actually using a real terminal.

Some KERMIT implementations may allow logging of the terminal emulation session
to a local file.  Such a facility allows "capture" of remote typescripts and
files, again with no error checking or correction.  When this facility is
provided, it is also desirable to have a convenient way of "toggling" the
logging on and off.

If the local system does not provide system- or firmware-@|level flow control,
like XON/XOFF, the terminal emulation program should attempt to simulate it,
especially if logging is being done.

The terminal emulation facility should be able to handle either remote or local
echoing (full or half duplex), any required handshake, and it should be able to
transmit any parity required by the remote side or the communication medium.

A terminal emulator works by continuously sampling both console input from the
local terminal and input from the communication line.  Simple input and output
functions will not suffice, however, since if you ask for input from a certain
device and there is none available, you will generally hang until input
@i<does> become available, during which time you will be missing input from the
other device.  Thus you must have a way to bounce back and forth regardless of
whether input is available.  Several mechanisms are commonly used:
@Begin(Itemize)
Continuously jump back and forth between the port status register and the
console status register, checking the status bits for input available.  This
is only practical on single-@|user, single-@|process systems, where the CPU has
nothing else to do.

Issue an ordinary blocking input request for the port, but enable interrupts on
console input, or vice versa.

Handle port input in one process and console input in another, parallel
process.  The UNIX KERMIT program listed in this manual uses this method.
@End(Itemize)
Any input at the port should be displayed immediately on the
screen.  Any input from the console should be output immediately to the port.
In addition, if the connection is half duplex, console input should also be
sent immediately to the screen.

The terminal emulation code must examine each console character to
determine whether it is the "escape character".  If so, it should take the next
character as a special command, which it executes.  These commands are
described above, in section @ref<-emulation>.

@Chapter<Writing a KERMIT Program>

@Index<Program, Kermit>
A basic KERMIT implementation can usually be written as a relatively small
program, self-contained in a single source file.  However, it is often the case
that a program written to run on one system will be adapted to run on other
systems as well.  In that case, it is best to avoid having totally divergent
sources, because when new features are added to (or bugs fixed in) the
system-@|independent parts of the program -- i.e. to the protocol itself --
only one implementation will reap the benefits initially, and the other will
require painful, error-@|prone "retrofitting" to bring it up to the same level.

Thus, if there is any chance that a KERMIT program will run on more than one
machine, or under more than one operating system, or support more than one kind
of port or modem, etc, it is desirable to isolate the system-@|dependent parts
in a way that makes the common parts usable by the various implementations.
There are several approaches:
@Begin(Enumerate)
Runtime support.  If possible, the program can inspect the hardware or inquire
of the system about relevant parameters, and configure itself dynamically at
startup time.  This is hardly ever possible.

Conditional compilation (or assembly).  If the number of systems or options to
be supported is small, the system dependent code can be enclosed in conditional
compilation brackets (like @q<IF WIDGET-8000 .... ENDIF>).  An example is
provided in UNIX KERMIT listing included with this manual.  However, as the
number of system dependencies to be supported grows, this method becomes
unwieldy and error-@|prone -- installing support for system X may well break
the pre-existing support for system Y.

Modular composition.  When there is a potentially large number of options a
program should support, it should be broken up into separate modules (source
files), with clearly specified, simple calling conventions.  This allows people
with new options to provide their own support for them in an easy way, without
endangering any existing support.  Suggested modules for a KERMIT program are:

@Begin(Itemize)
System-Indendent protocol handling: state switching, packet formation,
encoding (prefixing) and decoding, etc.

User Interface: the command parser.  Putting this in a separate module allows
plug-in of command parsers to suit the user's taste, to mimic the style of the
host system command parser or some popular application, etc.

Screen i/o: Allows various terminals to be supported; this module
would contain the screen control codes, cursor positioning routines, etc.

Port i/o: Allows support of various port hardware.  This module can define the
port status register location, the status bits, and so forth, and can implement
the functions to read and write characters at the port.

Modem control: This module would support any kind of "intelligent" modem, which
is not simply a transparent extension of the communications port.
Such modems may accept special commands to perform functions like dialing out,
redialing a recent number, hanging up, etc.

Console input: This module would supply the function to get characters from the
console; it would know about the status register locations and bits, interrupt
structure, key-to-@|character mappings, etc.

File i/o: This module contains all the knowledge about the host system's file
structure; how to open and close files, perform "get next file" operations,
read and write files, determine and set their attributes, detect the end of a
file, and so forth, and provides the functions, including buffering, to get a
character from a file and put a character to a file.  This module may also
provide file management services for local files -- directory listings,
deleting, renaming, copying, and so forth.

Terminal emulation: This module would provide terminal emulation by calling
upon the other system-@|dependent modules.

Definitions and Data: Separate modules might also be kept for compile-@|time
parameter definitions and for global runtime data.
@End(Itemize)
@End(Enumerate)
@Index<Language, Programming>
The language to be used in writing a KERMIT program is more than a matter of
taste.  The primary consideration is that the language provide the necessary
functionality and speed.  For instance, a microcomputer implementation of BASIC
may not allow the kind of low-@|level access to device registers needed to do
terminal emulation, or to detect console input during file transfer, or even if
it can do these things, it might not be able to run fast enough do drive the
communication line at the desired baud rate.

The second consideration in choosing a language is portability.  This is used
in two senses: (1) whether the language public domain (or, equivalently,
provided "free" as part of the basic system), and (2) whether it well
standardized and in wide use on a variety of systems.  A language that is
portable in both senses is to be preferred.

A new KERMIT program should be thoroughly documented; one of the hallmarks of
KERMIT is its documentation.  The documentation should be at both the user
level (how to use the program, what the commands are, etc, similar to the
documentation presently found in the @i<Kermit Users Guide>), and the
implementation level (describe system dependencies, give pointers for adapting
to new systems, and so forth).

@Index<Bootstrap>
Finally, a bootstrap procedure should be provided.  KERMIT is generally
distributed on magnetic tape to large central sites; the users at those sites
need ways of "downloading" the various implementations to their micros and
other local systems.  A simple bootstrap procedure would consist of precise
instructions on how to accomplish an "unguarded" capture of the program.
Perhaps a simple, short program can be written for each each end that will do
the job; listings and instructions can be provided for the user to type in and
run these programs.

@appendix<Packet Types>

For reference, here is a complete list of all the defined KERMIT packet types
and subtypes.  Required types are marked with an asterisk (*).

 @begin<description,spread 0, leftmargin +4, indent -4>
Y*@\Acknowledge (ACK)

N*@\Negative acknowledge (NAK)

S*@\Send initiate (exchange parameters)

I@\Initialize (exchange parameters).

F*@\File header

A@\File attributes

D*@\Data packet

Z*@\End of file (EOF)

B*@\Break transmission (EOT)

E*@\Error

R@\Receive Initiate (ask the server to send the specified files).

C@\Host Command.

G@\Generic Kermit Command.  Subcommands:
 @begin<description,spread 0, leftmargin +4, indent -4>
I@\Login

C@\CWD, Change Working Directory

L@\Logout, Bye

F@\Finish

D@\Directory

U@\Disk Usage Query

E@\Erase, Delete

T@\Type

R@\Rename

K@\Copy

W@\Who's logged in?

M@\Short Message

H@\Help

Q@\Server Status Query
@end<description>
@end<description>

@appendix<List of Features>

There's no true linear scale along which to rate Kermit implementations.
A basic, minimal implementation provides file transfer in both directions, and,
for microcomputers (PC's, workstations, other single user systems), terminal
emulation.  Even within this framework, there can be variations.  For instance,
can the program send a @i<file group> in a single command, or must a command be
issued for each file?  Can it time out?  Here is a list of features that may be
present; for any KERMIT implementation, the documentation should show whether
these features exist, and how to invoke them.

@Begin(Itemize) 
File groups.  Can it send a group of files with a single command, using
"wildcard", pattern, or list notation?  Can it successfully send or receive a
group of files of mixed types?  Can it recover from an error on a particular
file and go on to the next one?  Can it keep a log of the files involved
showing the disposition of each?

Filenames.  Can it take action to avoid overwriting a local
file when a new file of the same name arrives?  Can it convert filenames to and
from "normal form"?

File Attributes.  Can transfer binary as well as text files?  Can it send and
receive Attribute packets?

8th-Bit prefixing.  Can it send and receive 8-bit data through a 7-bit channel
using the prefixing mechanism?

Repeat-Count processing.  Can it send and receive data with repeated characters
replaced by a prefix sequence?

Terminal Emulation.  Does it have a terminal emulation facility?  Does it
provide various communication options, such as duplex, parity, and handshake
selection?  Can it transmit all ASCII characters?  Can it transmit BREAK?
Can it log the remote session locally, to provide unguarded file capture?

Communications Options.  Can duplex, parity, handshake, and line terminator be
specified for file transfer?

Block Check Options.  In addition to the basic single-@|character checksum, can
the two-@|character checksum and the three-@|character CRC be selected?

Basic Server.  Can it run in server mode, accepting commands to send or receive
files, and to shut itself down?

Advanced Server.  Can it accept server commands to delete files, provide
directory listings, send messages, and forth?

Issue Commands to Server.  Can it send commands to a server, and handle all
possible responses?

Host Commands.  Can it parse and send remote "host commands"?  If it is a
server, can it pass these commands to the host system command processor and
return the results to the local user Kermit?

Interrupt File Transfers.  Can it interrupt sending or receiving a file?  Can
it respond to interrupt requests from the other side?

Local File Management Services.  Are there commands to get local
directory directory listings, delete local files, and so forth?

File Attributes.  Can it send file attribute information about local files, and
can deal with incoming file attribute information?

Debugging Capability.  Can packet traffic be logged or examined?
@end<itemize>


@Appendix<A KERMIT Program>
@label<-KProg> @Index[Program, Kermit]

What follows is a listing of a real production version of KERMIT, written in
the C language@foot[Kernighan & Ritchie, @i<The C Programming Language>:
Prentice-@|Hall (1978)].  It is designed to run under various versions of the
UNIX operating system.  The basic KERMIT functionality is present, but very
little in the way of optional features.  Only the most rudimentary
(UNIX-@|style) command parser is provided.  The program illustrates many of
the considerations described in this manual, with respect to the protocol, the
program design, and so forth.

It must be emphasized that this is a bare minimum implementation of Kermit.
Anyone writing a new Kermit from scratch is encouraged to look at the source
for one of the more advanced implementations as a model (check the @i<Kermit
Users Guide> for a list of KERMIT implementations and their features).
Although you may not understand the language it's written in, there should be
profuse comments that can be useful.

The following program uses decimal notation for numbers, and @q<tochar()>
rather than @q<char()> for integer-@|to-@|character conversion.
@Begin<ProgramExample,blanklines hinge>
@Include<KER:KERMIT.C>
@End<ProgramExample>

@Include<KER:ASCII.MSS>

@case<device,x9700=`@COMMENT<BEGIN DUPLEX KLUDGE>
  @SendEnd(#Index "@begin<Transparent,PageBreak UntilOdd>@end<Transparent>")
  @COMMENT<END DUPLEX KLUDGE>
'>
