	PROGRAM SETUIC
C	IDENT /2.2/
C
C	SETUIC is used to change or display the UIC of a process.
C	It is designed to be used as a Foreign Command, ie
C
C		SETUIC:==$Dev:[directory]SETUIC
C
C	To be used by any user without CMKRNL privileges, it must be
C	installed with CMKRNL privilege.
C
C	Depending on the privilege of the user SETUIC will allow the
C	UIC to be changed in different ways.
C
C	1.  No privileges     -> Only may display UIC.
C	2.  Group Privilege   -> Only may change member number
C	3.  World Privilege   -> May change group and world numbers if
C				 group number > ggg but not 340 (admin)
C	4.  System Privilege  -> May change group and world numbers anywhere.
C
C		SETUIC	->  UIC = [ggg,mmm]
C
C		SETUIC [ggg,mmm]
C	or
C		SETUIC ggg,mmm
C
C	To build:
C		  $MACRO UICSET
C		  $FORTRAN SETUIC
C		  $LINK SETUIC,UICSET,SYS$SYSTEM:SYS.STB/SEL
C
C	Written by:	James G. Downward
C			KMS Fusion, Inc.
C			P.O. Box 1567
C			Ann Arbor, Mich 48104
C			(313)-769-8500
C
C		        Based on code origionally written by Mark Pilant.
C			
C
C Modified:	V2.1	James G. Downward	No change to ADMIN accounts
C						allowed except with SYSPRV
C-
C
	CHARACTER*3 Group,Member			!
	CHARACTER*15 UIC,XTRACT				!
	CHARACTER*23 Cprompt				!
	INTEGER*2 GNUM,MNUM,IUICVEC(2)			!
	INTEGER*2 JPI_Code,JPI_Length,JPI_Header(2)	! For GETJPI setup
	INTEGER*4 LIST(3), LENGTH			!
	INTEGER*4 Istatus,Iuser_priv,Ilength		!
	INTEGER*2 Iprocess_UIC(2),Iprocess_Group,Iprocess_Member
	INTEGER*4 LIB$GET_FOREIGN			!
	EXTERNAL  LIB$GET_FOREIGN			!

	EQUIVALENCE (JPI_Header(1),JPI_Length)
	EQUIVALENCE (JPI_Header(2),JPI_Code)
	EQUIVALENCE (JPI_Header(1),LIST(1))
	EQUIVALENCE (Iprocess_UIC(2),Iprocess_Group)
	EQUIVALENCE (Iprocess_UIC(1),Iprocess_Member)
	EQUIVALENCE (IUICVEC(1),MNUM),(IUICVEC(2),GNUM)

	Minimum_group='300'O				! Break point
	Iadmin_group='340'O				! Administration
c	Cprompt='Enter UIC ([ggg,mmm]): '		! Prompt for cmd

c	ISTATUS=LIB$GET_FOREIGN(UIC,Cprompt,LEN)	! Get command line
	ISTATUS=LIB$GET_FOREIGN(UIC,,LEN)		! Get command line
c	IF (LEN.LT.1) CALL EXIT 			! Exit if <cr>
	IF (LEN.LT.1) Idisplay=1			! Display only
C
C -- Find out the user's current privileges (world, group?)
C    And the starting UIC of the user
C
	JPI_Code = '204'X				! Get proc priv code
	JPI_Length = 4					! 4 bytes long
	LIST (2) = %LOC(Iuser_Priv)			! Set up GETJPI
	LIST (3) = %LOC(Ilength)			! 
	ISTATUS=SYS$GETJPI(,,,LIST,,,)			!
	JPI_Code = '304'X				! Get process UIC code
	LIST(2) = %LOC(Iprocess_UIC)			!
	Istatus=SYS$GETJPI(,,,LIST,,,)			!
C
C -- If display only, do it
C
	IF (Idisplay.EQ.1)		THEN		! If display only
	   WRITE(5,5)Iprocess_Group,Iprocess_Member	!  then show the UIC
5	   FORMAT(' UIC = [',O3.3,',',O3.3,']')		!  and exit
	   CALL EXIT					!
	END IF						!

C	WRITE(5,99)IUSER_PRIV
C99	FORMAT(' 'O12)
C -- Find out just how privileged the user is
C
	Igroup_Priv =IAND('400'O,Iuser_Priv)		! Group Privilege?
	Iworld_Priv =IAND('200000'O,Iuser_Priv)		! World Privilege
	Isystem_Priv=IAND('2000000000'O,Iuser_Priv)	! System Privilege
	Ipriv=Isystem_Priv+Iworld_Priv+Igroup_Priv	!
	IF(Ipriv.EQ.0)				THEN	! Must have one or
	  TYPE *, 'SETUIC -- Privilege violation.',	!  priv. violation
     -	          ' Group, World or SYSPRIV privilege required.'
	  CALL EXIT
	END IF

C
C	CHECK FOR VALID STRING CONTEXT

	DO 10 I=1,LEN
	   IF (UIC(I:I).EQ.' ')   		THEN	!
	     TYPE *,'SETUIC -- Invalid UIC.  No embedded blanks, please.'
	     CALL EXIT					!
	   END IF
10	CONTINUE
C

C
C	CHECK FOR A [					!
	IF (INDEX(UIC,'[').NE.0) 	GOTO 20		! Got a '[', need a ]
C							!
C	No [ so must not have a ]			! Check for matching 
	IF (INDEX(UIC,']').NE.0)       		THEN	!   [,] pairs
	  TYPE *,'SETUIC -- Invalid UIC string. "]" illegal if "[" omitted.'
	  CALL EXIT					!  Exit after warning
	END IF						! 
	GOTO 40						! UIC is ok 
C							!
C	Had a [ must have a ]				!
20	IF (INDEX(UIC,']').EQ.0)       		THEN	!
	  TYPE *,'SETUIC -- Invalid UIC string. "]" required if "[" present.'	
	  CALL EXIT					!
	END IF

C							!
C	STRIP OFF [ ]					!
	L=0						!
	XTRACT(1:7)='       '				!
	DO 30 J=2,LEN-1					!
		L=L+1					!
		XTRACT(L:L)=UIC(J:J)			!
30	CONTINUE					!
	UIC(1:7)=XTRACT(1:7)				!
	LEN=L						!
C
C	Break apart into member and group		!
40	K=INDEX(UIC,',')				! Is a comma there
	IF (K.EQ.0) 			THEN		!
	  TYPE *, 'SETUIC -- Invalid UIC. Comma missing in [ggg,mmm].'
	  CALL EXIT					!
	END IF						!
42	GROUP(1:K-1)=UIC(1:K-1)
	IF (K.EQ.2) GROUP='00'//GROUP			! Concatinate strings
	IF (K.EQ.3) GROUP='0'//GROUP			!
	L=LEN-K						!
	MEMBER(1:L)=UIC(K+1:LEN)			!
	IF (L.EQ.1) MEMBER='00'//MEMBER			!
	IF (L.EQ.2) MEMBER='0'//MEMBER			!
C
C	Put UIC back together in full 7 digit form	!
	UIC=GROUP//','//MEMBER				!
	LEN=7						! Final length
C

C
C	THIS REQ OK SO DO UIC DELTA
C
C	Convert UIC to numeric entry, check to be sure OCTAL
80	GNUM=0
	MNUM=0
	DO 90 I=1,3
	   IF (GROUP(I:I).LT.'0'	.OR.		!
     -	       GROUP(I:I).GT.'7'	.OR.		!
     -	       MEMBER(I:I).LT.'0'	.OR.		!
     -	       MEMBER(I:I).GT.'7')		THEN	!
	       TYPE *,'SETUIC -- Invalid UIC. Octal numbers only.'
	       CALL EXIT				!
	   END IF					!

	   GNUM=GNUM*8+(ICHAR(GROUP(I:I))-'30'X)	! Convert to numeric
	   MNUM=MNUM*8+(ICHAR(MEMBER(I:I))-'30'X)	!  value
90	CONTINUE

C
C -- At this point we know the user is either a group or world manger.
C    See if what they want to do is OK.  If not warn them and exit
C
	IF(Isystem_Priv.GT.0)		GOTO 300	! User can do anything
	IF(Iworld_Priv.EQ.0)		THEN		! User is group mngr
	  IF(Iprocess_Group.NE.GNUM)	THEN		!  May only change 
	    WRITE(5,200)				!   the member code
200	    FORMAT(' SETUIC -- Privilege Violation.',	!
     -             '  Group code may not be changed.')	!
	    CALL EXIT					!  And exit
	  END IF					!
	END IF						!

	IF(Iworld_Priv.GT.0 		.AND. 		!
     -	  GNUM.LT.Minimum_group) 		THEN	! World Manager may
	  WRITE(5,220) Minimum_Group			! not change UIC to
220	  FORMAT(' SETUIC -- Privilege Violation.',	! be <=277
     -         '  Group code must be =>',O3)		!
	  CALL EXIT					! And exit
	END IF
	IF(Iworld_Priv.GT.0 		.AND. 		!
     -	  GNUM.EQ.Iadmin_Group) 		THEN	! World Manager may
	  WRITE(5,230) 					! not change UIC to
230	  FORMAT(' SETUIC -- Privilege Violation.',	! be 340
     -         '  Access to administration accounts denied.')	!
	  CALL EXIT					! And exit
	END IF
300	CONTINUE
	CALL UICSET (IUICVEC)
	CALL EXIT (1)
	END
