	PROGRAM BROADCAST
C+
C		IDENT /1.6/
C
C	This program function is to provide the functionality of the
C	RSX BROADCAST command to all users.  For non privileged users
C	to use it it must be installed with OPER privileges.  It mus
C	be defined as a foreign command, ie
C
C		BROADCAST:==$SYS$SYSDEVICE:[SYSUTL]BROADCAST
C
C	The command syntax allows command lines of the form
C
C		BROADCAST TTxx:messagemessagemessage
C	or
C		BROADCAST Username:messagemessagemessage
C
C	First BROADCAST checks for the ":", if non is found it returns
C	an error.  If one is found it uses the first two letters of the
C	command line to decide if it is a terminal or a username.  If the
C	first two letters are "TT", it decides it is a termial, else it
C	assumes it is a username.
C
C	For BROADCAST to work for all users it must be installed with
C	OPER and WORLD privileges. ie
C
C		MC INSTALL
C		INSTALL> BROADCAST/PRIVILEGE=(OPER,WORLD)
C		INSTALL> ^Z
C
C	As currently implemented, BROADCAST will erases the top 6  lines
C	of a DEC CRT screen to display its message and beep as the message
C	is delivered.
C
C
C	Written by:	James G. Downward
C			KMS Fusion, Inc.
C			P.O. Box 1567
C			Ann Arbor, Mich 48104
C			(313)-769-8500
C			10-Oct-1982
C
C	Modified by:
C			James G. Downward	11-Nov-1982	V1.1
C						Exit with informational status
C						if user not found.
C
C			James G. Downward	28-Nov-1982	V1.2
C						Fix problems with quotes and
C						Broadcasts to user on multiple
C						terminals
C
C			James G. Downward	21-Jan-1982	V1.3
C						Fixed incorrect length for user
C						name if broadcasting from a 
C						user with name shorter than
C						name of the other user.
C			
C			James G. Downward	21-Jan-1982	V1.4
C						Don't broadcast to batch jobs
C						particularly if they started
C						us up.
C
C			James G. Downward	12-Arp-1983	V1.5
C						Allow broadcasts to OPA0:
C
C			James G. Downward	20-Sep-1983	V1.6
C						Allow user names to be
C						logical names pointing
C						to a single user name. IE
C						VAX$MANAGER -> Downward
C						Logical names up to 31 char.
C-
C



	CHARACTER*7	Terminal			!
	CHARACTER*12	Username			!
	CHARACTER*31	Logical_Username		! Maybe a logical name
	CHARACTER*132	Command_Line			!
	Character*132	Broadcast_message		!
	LOGICAL*1	Ionce,Ifound,BTEST		!
	INTEGER*2	JPI_Code,JPI_Length,JPI_Header(2)! For GETJPI setup
	INTEGER*4 	LIST(10)			! GETJPI List
	INTEGER*4 	Istatus,Ulength,Tlength		!
	INTEGER*4	Istatus_Flags			! Job status flags
	INTEGER*4 	SYS$GETJPI			! Get Job Param
	INTEGER*4 	LIB$GET_FOREIGN			!
	INTEGER*4	SYS$TRNLOG			! Translate log names
	INTEGER*4 	SYS$EXIT			!
	INTEGER*4 	SYS$BRDCST			!
	INTEGER*4	Iflag				!
	INTEGER*4 	PIDADR				! Process PID
	INTEGER*4 	Ihighbits			!
	INTEGER*4 	Ists				!


	EQUIVALENCE (JPI_Header(1),JPI_Length)		!
	EQUIVALENCE (JPI_Header(2),JPI_Code)		!
	EQUIVALENCE (JPI_Header(1),LIST(1))		!
	INCLUDE '($SSDEF)'				! Signal Definitions
	INCLUDE '($JPIDEF)'				! GETJPI Definitions
	INCLUDE 'PCBDEF.FOR'				! For PCB$V_BATCH

	PARAMETER BRDCST$M_SCREEN= '00000100'X		! From $BRDCSTDEF
	PARAMETER BRDCST$M_BOTTOM= '00000200'X		!
	PARAMETER BRDCST$M_REFRSH= '00000400'X		!
	PARAMETER (Ihighbits='2000000000'O)		! The bit to set to
							! inhibit error msg

	Iflag=BRDCST$M_SCREEN.or.6			! Erase top 6 lines

	ISTATUS=LIB$GET_FOREIGN(Command_Line,,Length)	! Get command line
	IF (Length.LT.1) CALL EXIT 			! Exit if <cr>
	Ipos=INDEX(Command_line,':')			! Find the ":"
	IF(Ipos.EQ.0)				THEN	! If not found
	   WRITE(*,10)					!   then warn user
10	   FORMAT(' BROADCAST -- Fatal, No ":" seperator found')! 
	   Ists=Ihighbits+2				! Set high bits
	   Istatus=SYS$EXIT(%VAL(Ists))			! Exit with bad status
	END IF


	Ionce=.TRUE.					! Only scan once
	Ipos2=Ipos					! Copy position
	IF(Command_Line(1:2).EQ.'TT'	.OR.		! Go directly to term
     -     Command_Line(1:3).EQ.'_TT'	.OR.		!
     -	   Command_Line(1:4).EQ.'OPA0')		THEN	!
	  Terminal(1:ipos)=Command_Line(1:Ipos)		! Transfer term
	  Ifound=.TRUE.					! Always find terminal
	  GOTO 1000					! Access by terminal
	END IF

200	PIDADR=-1					! Check all proc
	Ionce=.FALSE.					! Repeat till done
C
C	We must have a user name not a terminal
C --    Scan and fine all users who match the user name.  First set up
C	the control block for GETJPI, proceeding in a backwards manner.
C	Remember the last element in the array must be zero to indicate
C	nothing more to process.
C
	JPI_Code = JPI$_STS				! Process status
	JPI_Length = 4					! An integer return
	List(7)  = List(1)				! Set up Get_Jpi header
	List(8)  = %LOC(Istatus_Flags)			! Where to put flag wd
	List(9)  = 0					! no length returned
	JPI_Code = JPI$_TERMINAL			! Set up the GETJPI
	JPI_Length = 7 					!  list for terminal
	LIST(4)  = LIST(1)				!  as second element
	LIST(5)  = %LOC(Terminal)			!  of list
	LIST(6)  = %LOC(Tlength)			!
	JPI_Code = JPI$_USERNAME			! Get username
	JPI_Length = 12					! 12 bytes long
	LIST (2) = %LOC(Username)			! Set up GETJPI
	LIST (3) = %LOC(Ulength)			! 
C
C --    GETJPI control block now set up, but first try and translate the
C	name (all that stuff up to the ":") from a logical name to a 
C	real name.  If not logical name, this procedure will not change it
C	and the name will be transfered untranslated.
C
	Istatus=SYS$TRNLOG(Command_Line(1:Ipos-1),Itrans_Length,
     -                     Logical_Username,,,)		!
	CALL SYS$CLREF(%VAL(1))				! Clear event flag
300	Istatus=SYS$GETJPI(%VAL(1),PIDADR,,LIST,,,)	! Get info
	IF(Istatus.EQ.SS$_NOMOREPROC)		THEN	! Repeat until no more
	  IF(.NOT.Ifound) 			THEN	! If user is never
	    WRITE(*,310)Command_Line(1:Ipos-1)!		!  found, warn with
310	    FORMAT(' BROADCAST -- User ',A,' is not on the system') ! msg
	    Ists=Ihighbits+2				! Set high bits
	    Istatus=SYS$EXIT(%VAL(Ists))		! Exit with bad status
	  END IF					!
	  CALL EXIT					! Else exit, found user
	END IF						!
	CALL SYS$WAITFR(%VAL(1))			! Wait till we have it
	CALL SYS$CLREF(%VAL(1))				! Clear efn

C
C  --	Since BROADCASTING a message is designed to return Success status
C	if a user is found, a batch job should not be allowed to fool the
C	program into thinking the user has seen the message.  Virtual
C	users do not count.
C
	IF(BTEST(Istatus_Flags,PCB$V_BATCH)) GOTO 300	! no brdcst to batch
	
C -- If not the specified user, try, try again
C	IF(Command_Line(1:Ipos-1).NE.Username(1:Ulength)) GOTO 300
	IF(Logical_Username(1:Itrans_Length).NE.Username(1:Ulength))
     -						GOTO 300! 
	   
	Ipos2=Tlength					! Ok, use new length
	IF(Ipos2.EQ.0)				GOTO 300! Try again if null
1000	Broadcast_Message(1:1)=CHAR('12'O)		! Place display 2
	Broadcast_Message(2:2)=CHAR('12'O)		! line from top of
	Broadcast_Message(3:3)=CHAR('12'O)		! DEC CRT terminal
	Broadcast_Message(4:4)=CHAR(7)			! ring bell
	IF(Command_Line(Length:Length).EQ.'"')Length=Length-1! Throw away "
	I=Ipos						! save ":" position
	IF(Command_Line(Ipos+1:Ipos+1).EQ.'"')I=I+1	! Disgard quote after :
	Broadcast_Message(5:)=Command_Line(I+1:Length)	! Put in message
	Istatus=SYS$BRDCST(Broadcast_Message,Terminal(1:Ipos2),
     -			   %VAL(Iflag),)

c	WRITE(*,1010) IPOS2,Terminal(1:ipos2),BROADCAST_MESSAGE(5:40)
c1010	format(' Ipos2=',i5,'Terminal = *',a,'*   ', 'Message =*',A,'*')
C ...
C ... Documentation for condition passing states that for signaling
C ... exit status without sending a message, one should set bit 28
C ... in the I*4 condition value as follows
C ...
C ...		Bit 28  '2000000000'O   Suppress error message
C ...
C
	IF(Istatus.EQ.SS$_Normal)		THEN	!
		Ifound=.TRUE.				! Show user found
		IF(.NOT.IONCE) 	GOTO 300		! Check all users
		CALL EXIT				! Else exit
	ELSE IF(Istatus.EQ.SS$_BADPARAM)	THEN	!
	        WRITE(*,9000)				! Warn, msg too long
	        GOTO 9999				! Exit
	ELSE IF(Istatus.EQ.SS$_DEVOFFLINE)	THEN	!
		WRITE(*,9010)Terminal			! Device not term or
		GOTO 9999				!  offline
	ELSE IF(Istatus.EQ.SS$_NONLOCAL)	THEN	! Device not local
		WRITE(*,9020)				!
		GOTO 9999				!
	ELSE IF(Istatus.EQ.SS$_NOPRIV)		THEN	! If not enough priv
		WRITE(*,9030)				!
		GOTO 9999				!
	ELSE IF(Istatus.EQ.SS$_NOSUCHDEV)	THEN	! If no such device
		WRITE(*,9040) Terminal			!
		GOTO 9999				!
	ELSE						! Last chance err trap
		WRITE(*,9050)Istatus			!
		GOTO 9999				!
	END IF
	IF (.NOT. Ionce)	GOTO 300		! Repeat for all proc
9000	FORMAT(' BROADCAST -- Fatal, Message too long')
9010	FORMAT(' BROADCAST -- Fatal, Device ',A,' not a terminal',
     -	       ' or offline')
9020	FORMAT(' BROADCAST -- Warning, Device on remote node')
9030	FORMAT(' BROADCAST -- Fatal, Insufficient privilege to broadcast')
9040	FORMAT(' BROADCAST -- Fatal, Terminal ',A,' does not exist or',
     -	       ' can not receive messages')
9050	FORMAT(' BROADCAST -- Fatal, Istatus = ',z10)	!
9999	CONTINUE
	IF(.NOT.Ionce)				GOTO 300! Repeat till done
	Ists=Ihighbits+2				! Set high bits
	Istatus=SYS$EXIT(%VAL(Ists))			! Exit with status
	END


