C+
C
C Display_Record
C
C	This routine will display (to the terminal), the data record
C	in common, using the form Iform, and the order and number of
C	items to print stored in the FORM common.  
C
C		CALL Display_Record(Iform [,Lun])
C	Where 
C			Iform	I*4	Form number
C			Lun	I*4	Logical Unit Number for display
C					Optional.  
C					If not present -> SYS$OUTPUT
C					If present, file must be opened.
C
C	Very little input data is supplied to this routine because
C	the required data should already exist in COMMON.  The data
C	is placed in common by LODFORM and LODPRMPTS.  This routine
C	is a bit tricky.  Cform contains a format statement loaded
C	from the text library.  It can be 132 bytes long.  The order
C	of printing of the fields in RECORD is obtained from the
C	Iorder_of_Items array which is dimensioned IsizeXIsize.
C	For a given form, Iorder_of_Items(Iform,I) contains the
C	number of the next field to print, if the number is 0,
C	no field is there to print.  The number of items to print
C	is contained in Number_of_Items.  So the starting and ending
C	locations for the field numbers from Iorder_of_Items(Iform,1)
C	to Iorder_of_Items(Iform,Number_of_Items) is obtained and
C	stored in the arrays Ibegin() and Iend().
C
C	Required commons:
C
C		COMMON	/Prompts/
C		COMMON	/Forms/
C		COMMON	/Record_Buf/
C
C	Written by:
C			James G. Downward
C			KMS Fusion, Inc
C			PO Box 1567
C			Ann Arbor, Mich. 48106
C			12-Jan-1982
C
C			
C+

	SUBROUTINE Display_Record(Iform,Lun)
C
C		IDENT /1.3/
C
	PARAMETER		(Isize=20)		! Max # of items

	CHARACTER*50		Cfield_name		! Prompt string
	CHARACTER*50		Cfield_Help		! Help string
	CHARACTER*50		Cmatch_String		! Match string
	CHARACTER*50		Cform_Name		! Form name
	CHARACTER*512		Record			! the data buffer
	CHARACTER*132		Cform			! the form buffer
	Integer*2		Ibegin(Isize)		! Field loc in rec
	Integer*2		Iend(Isize)		! Field end in rec
	INTEGER*4		Field_name_length	! Length of prompt
	INTEGER*4		Field_Help_Length	! Length of HELP string
	Integer*4		Field_length		! Length of input data
	Integer*4		Number_of_Entries	! Number of prompts
	INTEGER*4		Number_of_Forms		! Max number of forms
	INTEGER*4		Number_of_Items		! # of items to print
	INTEGER*4		Iorder_of_Items		! Order to print items
	INTEGER*4		Record_Length		! Length of data rec
	INTEGER*4		Match_type		! Type of match to chk

	COMMON	/PROMPTS/	Number_of_Entries,	! Number of prompts
     -				Field_name_length(Isize),! Length of prompt
     -				Field_Length(Isize),	!  Length of input
     -				Field_Help_Length(Isize),! Length of help str
     -				Match_type(Isize),	!  Type of match to chk
     -				Cfield_name(Isize),	!  Prompt string
     -				Cfield_HELP(Isize),	!  Help module name
     -				Cmatch_String(Isize)	!  String to match

	COMMON	/FORMS/		Number_of_Forms,	! # of forms input
     -				Number_of_Items(Isize),	! # of items to print
     -				Iorder_of_Items(Isize,Isize),! Order to print
     -				Cform(Isize),		! Storage for form
     -				Cform_name(Isize)	! Form name


	COMMON	/Record_buf/	Record_Length,Record	! storage for data rec

	CALL Argcnt(Narg)				! Optional arg present?
	IF(Iform.GT.Number_of_Forms)		THEN	! If the requested form
	  TYPE *,'VPW_Display_Record -- Illegal form number' ! is greater than
	  TYPE *,' '					! the maximum form
	  RETURN					! stored, ret/error
	END IF						!

	DO I=1,Number_of_Entries			! Fnd strt loc for each
	  Istart=1					!  Start with 1st char
	  DO J=1,Iorder_of_Items(Iform,I)-1		!  Scan the order array
	    IF(Iorder_of_Items(Iform,I).GT.1) 		!  If not null, add in
     -			Istart=Istart+Field_Length(J)	!    each field length
	  END DO					!    to find start char
	  K=Istart+Field_Length(Iorder_of_Items(Iform,I))-1! get ending char
	  CALL STRING_LENGTH(Record(Istart:K),Ilength)	!  find real length
	  Ibegin(I)=Istart				!  set beginning char
	  Iend(I)=Istart+Ilength-1			!  set ending char
	END DO						! for entry
c	Iend(Number_Of_Entries)=Record_Length		! For last entry
	IF(Narg.EQ.1)				THEN	! If only 1 arg
	  WRITE(*,Cform(Iform))				! Now write it out
     -		(Record(Ibegin(I):Iend(I)),I=1,Number_of_Items(Iform))
	ELSE						! If two arg
	  WRITE(Lun,Cform(Iform))			! Now write it out
     -		(Record(Ibegin(I):Iend(I)),I=1,Number_of_Items(Iform))
	END IF						!

	RETURN						!
	END						!


