C
C File_Sort
C
C	This subroutine is used to sort ASCII format files according to
C	fields defined in the common block SORT.  Normally, the VPW
C	subroutine LODSORT, loads the common block, with the sort
C	information stored in SYS$VPWFILES:VPW.TLB in the appropriate
C	module.  By default record fault is used and if the sort is
C	successful, the old file is deleted and the new file kept.
C	If the sort fails the new file is deleted and the old kept.
C
C		CALL File_Sort(Cfile,Iterminal,Istatus)
c
C 	where
C		Cfile		C*(*)		File name to sort
C		Iterminal	I*2		Terminal type (>= 96=>VT100)
C		Istatus		I*2		Sort exit status
C						=  1  Success
C						= -1  Failure
C
C	Required Common Blocks:
C
C	CHARACTER*1	    Cesc,Cbell,Cnull	! Provides information for
C	CHARACTER*4	    Cbold, Cnorm	! terminal video attributes
C	INTEGER*2	    Key_Info		! Controls sort
C	COMMON	/SORT/	    Key_Info(41)	!  Room for 10 sort keys
C	COMMON	/Constants/ Cesc,Cbell,Cnull,	!
C			    Cbold,Cnorm		!
C
C	If it is not desired to sort, an immediate return is done
C	if Key_Info(1) = 0.  If files are to be sorted, the 
C	screen is erased, and a message displayed that sorting
C	is taking place.  The proper format for the constants
C	loaded into Key_Info is documented in LODSORT.FOR and
C	in the Sort manual.
C
C	For each key to	sort on, a block of 4 numbers (seperated with commas) 
C	must exist.
C
C	#_of_keys,Data_type,Ascend/Descend,Start_Pos,Length,......
C		  _________________________________________
C
C		   <-- the required 4 words of info/key -->
C
C	The data is returned in an I*2 array of dimension 41 which is passed
C	in the common block SORT.  Information on how to set up the blocks of
C	numbers may be obtained in the VAX SORT manual, Section 3.2.2.
C
C	The **BUG** fix relates is documented in the VAX March 1983 System
C	dispatch and allows TAG sorts to be used.
C	
C-
	SUBROUTINE File_Sort(Cfile,Iterminal,Istatus)

	IMPLICIT	INTEGER*4 (A - Z)		!
	CHARACTER*1	Cesc,Cbell,Cnull		!
	CHARACTER*4	Cbold,Cnorm			! Video attributes
	CHARACTER*(*)	Cfile				! Data file to use
	CHARACTER*30	In_File, Out_File		! In/out file names
	CHARACTER*132	Ctmp				! Temporary string
	BYTE		SORT_TYPE   /2/			! Tag sort
	BYTE		Work_FIles  /2/			! Number of work files
	BYTE		Ikey_Size   /50/		! Total key size
	BYTE		SOR$GB_SOR_TYP			! ** bug fix **
	LOGICAL*1	Lkill_New_File	       /.FALSE./! Do not delete output
	INTEGER*2	Key_Info			! Sort keys
	INTEGER*4	Isize	/10/			! Initial file size
	INTEGER*4	Ists				!
	EXTERNAL 	SOR$GB_SOR_TYP			! ** bug fix **

	COMMON	/Constants/	Cesc,Cbell,Cnull,Cbold,Cnorm
	COMMON	/SORT/	Key_Info(41)			! Sort Key_Info

	IF(Key_Info(1).EQ.0)			RETURN	! No Sorting to be done


	ILEN=LEN(CFILE)					! Length of file name

	CALL String_Length(Cfile,Ilen)			!
	Ctmp='Sorting file '//Cbold//Cfile(1:Ilen)//Cnorm!
	CALL String_Length(Ctmp,I)			!
	Ipos=(80-I)/2					!
	CALL PAGE					!
	CALL SCREEN(Ctmp(1:I),Iterminal,10,Ipos,0)	!


C ..
C ..	Open input and create output files
C ..
	Lkill_New_File=.FALSE.				! Do not del out file
	SOR$BG_SOR_TYP=SORT_TYPE			! Set TAG sort
	Status = SOR$PASS_FILES (Cfile, Cfile,,,,,,50)	! Set the file names
	IF (.NOT. Status) 		THEN		! If Sort init fails
	  WRITE(*,10) Cbold,Cfile(1:Ilen),Cnorm, Status	!
10	  FORMAT(' VPW_Sort_File -- File ',A,A,A,
     -		 ' could not be opened,',/,
     -           '                  Status = ',Z10)
	  GOTO 9900
	END IF

C	Describe key and initialize work area	!
	Status = SOR$INIT_SORT (KEY_INFO,,Isize,WORK_FILES,
     -			SORT_TYPE)		!
	IF (.NOT. Status) 		THEN	! If Sort init fails
	  WRITE(*,20) Status			!
20	  FORMAT(' VPW_Sort_File -- Work area initialization failed',/,
     -           '                  Please check current disk quota.',/,
     -		 '                  Sort Error Status = ',Z10)
	  GOTO 9900
	END IF

C	Sort records				!
	Status = SOR$SORT_MERGE ()		!
	IF (.NOT. Status) 		THEN	! If Sort init fails
	  WRITE(*,30) Status			!
30	  FORMAT(' VPW_Sort_File -- Files could not be sorted.',/,
     -     '                  Please check your disk quota allocation',/,
     -	   '                  Sort Error Status = ',Z10)
	  GOTO 9900
	END IF					!

C	Close files and clean up work area	!
	Status = SOR$END_SORT ()		!
	IF (.NOT. Status) 		THEN	! If Sort init fails
	  WRITE(*,40) Status			!
40	  FORMAT(' VPW_Sort_File -- Sorted files can not be closed',/,
     -		 '                  Sort Error Status= ',Z10)
	  GOTO 9900
	END IF					!

C ..
C ..	Keep the new file, delete the old one
C ..
	OPEN(	UNIT	= 1,			! Open new file
     -		NAME	= Cfile,		!
     -		TYPE	= 'OLD',		!
     -		DISPOSE = 'KEEP',		!
     -		ERR	= 8000)			! Never delete it

	OPEN(	UNIT	= 2,			! Open previous version
     -		NAME	= Cfile//';-1',		!
     -		TYPE	= 'OLD',		!
     -		DISPOSE = 'DELETE',		! Delete the blighter
     -		ERR	= 8010)			! If can't open file
	CLOSE(UNIT = 1,ERR=8020)		!
	CLOSE(UNIT = 2,ERR=8030)		! By, By old file
	Istatus=1				!
	RETURN					!

8000	Ctmp='VPW_Sort_File -- New data file '//Cbold//
     -	     Cfile//Cnorm//' can not be opened.'
	GOTO 8150				!
8010	Ctmp='VPW_Sort_File -- Old data file '//Cbold//
     -	     Cfile//Cnorm//' can not be opened for delete.'
	GOTO 8150				!
8020	Ctmp='VPW_Sort_File -- New data file '//Cbold//
     -	     Cfile//Cnorm//' can not be closed.'
	GOTO 8150				!
8030	Ctmp='VPW_Sort_File -- Old data file '//Cbold//
     -	     Cfile//Cnorm//' can not be closed for delete.'
	GOTO 8150				!

8040	Ctmp='VPW_Sort_File -- New data file '//Cbold//
     -	     Cfile//Cnorm//' can not be opened for delete.'
	Lkill_New_File=.TRUE.			! Try another way
	GOTO 8170				!
8050	Ctmp='VPW_Sort_File -- Old data file '//Cbold//
     -	     Cfile//Cnorm//' can not be opened.'
	GOTO 8170				!
8060	Ctmp='VPW_Sort_File -- New data file '//Cbold//
     -	     Cfile//Cnorm//' can not be closed for delete.'
	Lkill_New_File=.TRUE.			! Try another way
	GOTO 8170				!
8070	Ctmp='VPW_Sort_File -- Old data file '//Cbold//
     -	     Cfile//Cnorm//' can not be closed.'
	GOTO 8170				!

8150	CALL STRING_LENGTH(Ctmp,Ilength)	!
	WRITE(*,8160) Ctmp(1:Ilength)		!
8160	FORMAT(' ',A)				!
	GOTO 9990				!	

8170	CALL STRING_LENGTH(Ctmp,Ilength)	!
	WRITE(*,8180) Ctmp(1:Ilength)		!
8180	FORMAT(' ',A)				!
	GOTO 9990				!	

C ..
C ..	Delete the new file, keep the old one
C ..
9900	Status = SOR$END_SORT ()		! Close files
	OPEN(	UNIT	= 1,			! Open new file
     -		NAME	= Cfile,		!
     -		TYPE	= 'OLD',		!
     -		DISPOSE = 'DELETE',		!
     -		ERR	= 8040)			! the new one must go

	OPEN(	UNIT	= 2,			! Open previous version
     -		NAME	= Cfile//';-1',		!
     -		TYPE	= 'OLD',		!
     -		DISPOSE = 'KEEP',		! Keep it
     -		ERR	= 8050)			! If can't open file
	CLOSE(UNIT = 2,ERR=8070)		!
	CLOSE(UNIT = 1,ERR=8060)		! By, By new file

9990	Istatus=-1				!
	IF(Lkill_New_File)		THEN	! If new file must go
	   Ctmp='VPW_Sort_File -- Using VMS DELETE command to delete file '
     -	         //Cbold//Cfile//Cnorm//'.'	! Tell what we are doing
	   CALL STRING_LENGTH(Ctmp,Ilength)	!
	   WRITE(*,8180) Ctmp(1:Ilength)	!
	   Istatus=LIB$SPAWN('$DELETE '//Cfile//';0')! Kill the bugger
	END IF
	
	TYPE *, ' '				!
	TYPE *, 'Press '//Cbold//'RETURN'//Cnorm//' to continue'
	READ(*,9995,END=9999)Ctmp		!
9995	FORMAT(A)				!

9999	RETURN					!	

	END
