C+
C Get_Record
C
C	This subroutine uses the prompts stored in the PROMPTS common,
C	to prompt for and build up a data record in RECORD.  Four commons
C	must exist: PROMPTS, FORMS, CONSTANTS, and RECORD_BUF.  The Prompts
C	common must be loaded by LODPRMPT and the Forms common by LODFORMS.
C	The CONSTANTS common is loaded by the mainline and is used to
C	set video attirbutes for error messages.
C
C	CALL Get_Record(Iterminal,Imode,Iform,Istatus)
C
C Where
C	Iterminal	I*2	Terminal type (>= 96 is VT100 class)
C	Imode		I*2	Data entry mode
C				= 1  => Add mode    -- first field required
C				= 2  => Update mode -- all fields optional
C	Iform		I*2	Form number to use for data display
C	Istatus		I*2	Return status
C				= 1  => Success
C				= 0  => <CR> input to exit add mode
C				=-1  => ^Z input to exit from input
C
C	In "Add Mode", any entry field may be required to be non-null
C	(ie no carriage return).  In Update mode a Carriage Return is allowed.
C	If a carriage return is typed for a required entry in 'Add Mode', an
C	immediate return to the menu is made.  At any other time, all input
C	may be canciled by typing ^Z for any input.
C
C	If a carriage return is allowed, it is interpreted to mean, 'do not
C	change the contents of a field.  To null out a field, type a space.
C	As each entry is made, the record in its entireity is displayed
C	at the lower half of the screen.  Data entry occurs within
C	a scrolling region, so up to 20 items may be entered.  If one realizes
C	that an entered item is bad, one can hit the Up-Arror key on the
C	VT100 to back up one record.  Also allowed on any input are:
C
C		PF1	Keypad		Display the keypad keys
C		PF2	Help		Display Help info for current item
C		PF3	---		---
C		PF4	VPW>		Enter VPW command mode
C		7	Quit		Cancel current operation (line ^Z)
C		0	Menu		Quit and go to VPW menu
C		-	Done		Exit mode before last field
C		^	
C		|	Backup		Backup one record.
C
C	The form required for the text file which is used to load the
C	prompt string is defined in LODPRMPTS.FOR.  Prompt strings can
C	have up to a maximum of 50 characters.
C
C				Match Types
C
C	Match types are designed to be additive.  Consequently the numbers for
C	Match Types increase in binary order, 0,1,2,4,8...  A given input
C	may have more than one match type attribute but some of the types are
C	natually mutually exclusive.
C
C	None:		If the Match_type is 0, no match checking is done and 
C			the input is not required.
C	Exit on NULL:	If the input in add mode is null, a return to the
C			calling routine is done without entering data.
C			In Update mode, if the data is not present it
C			is treated as a no-op and the input data is read
C			for the next available field.
C	Required:	Required data must be entered in add mode.  If a
C			<CR> is entered a warning message is produced and
C			the data is again prompted for.  In update mode
C			no data is required.
C	Protected:	A protection field is a character string. Any location
C			of the string which contains other than a underline 
C			must be filled with only the character in that field.
C			Spaces containing an underline may be filled with any
C			character.
C	One of List:	Entries are matched against a list of strings
C			seperated by spaces.  The entry is accepted only
C			if there is an exact match.
C	Date:		Verify that the entered string is a valid VMS
C			date string.
C	Range check:	The input value is converted to a floating point
C			value and compared with the Lo/High values in the
C			string.  The format for storage is LOW/HIGH.  The
C			"/" seperates the values.  The entry is allowed
C			If and only if the entered value is >= Low value
C			and <= High value.
C
C	Definition of MATCH Types:
C	
C		Match_Type  =	0	Don't do match checking
C			    =	1	Exit on Null
C			    =	2	Parameter must be present
C			    =	4	Do not modify protected fields
C			    =	8	Match one of list
C			    =  16	Verify input is a date string
C			    =  32	Range check input
C
C
C	Written by:
C			James G. Downward
C			KMS Fusion, Inc
C			PO Box 1567
C			Ann Arbor, Mich. 48106
C			18-Jan-1983
C-

	SUBROUTINE Get_Record(Iterminal,Imode,Iform,Istatus)
C
C			IDENT 	/3.3/
C

	PARAMETER		(Isize=20)		!

	CHARACTER*1		Cesc,Cbell,Cnull	! For video attributes
	CHARACTER*1		cquote,Cans		!
	CHARACTER*4		Cbold,Cnorm		! For Video attributes
	CHARACTER*50		Cfield_name		! Prompt for input
	CHARACTER*50		Cfield_Help		! Help request string
	CHARACTER*50		Cmatch_String		! Match field
	CHARACTER*50		Cform_name		! Form name
	CHARACTER*80		Cprompt			! Prompt string
	CHARACTER*132		Ctmp			!
	CHARACTER*132		Cform			!
	CHARACTER*512		Record			! Data record

	LOGICAL*1		Error_Flag		! True if error occurs
	INTEGER*2		Imode			! 1/2 for Add/Update
	INTEGER*2		Imode_Add	   /1/	! Add mode
	INTEGER*2		Key_Info		! Sort information
	INTEGER*2		Imode_Update	   /2/	! Update mode
	INTEGER*2		Match_none	   /0/	! Do not match
	INTEGER*2		Match_Exit_on_Null /1/	! Exit if no input
	INTEGER*2		Match_Required	   /2/	! Must enter match
	INTEGER*2		Match_Protected	   /4/	! don't chng prot flds
	INTEGER*2		Match_List	   /8/	! Match one of list
	INTEGER*2		Match_Date	  /16/	! Verify input is date
	INTEGER*2		Match_Range	  /32/	! check range of input
	INTEGER*4		Field_name_length	! Length of prompt
	INTEGER*4		Field_Help_Length	! Length of help string
	Integer*4		Field_length		! Allowed input length
	Integer*4		Number_of_Entries	! # of fields in record
	INTEGER*4		Number_of_Forms		! Max # of form read in
	INTEGER*4		Number_of_Items		! # of fields to print
	INTEGER*4		Iorder_of_Items		! Order of print items 
	INTEGER*4		Record_Length		! Length of data rec
	INTEGER*4		Match_Type		! Type of matching
	INTEGER*4		Timbuf(2)		! for date check
	INTEGER*4		SYS$BINTIM		! for exit status
	REAL*4			Rval, Loval, Hival	! for range check
	
	COMMON	/Constants/	Cesc,Cbell,Cnull,Cbold,Cnorm ! Video attributes

	COMMON	/PROMPTS/	Number_of_Entries,	!
     -				Field_name_length(Isize),!
     -				Field_Length(Isize),	!
     -				Field_Help_Length(Isize),!
     -				Match_Type(Isize),	!
     -				Cfield_name(Isize),	!
     -				Cfield_HELP(Isize),	!
     -				Cmatch_String(Isize)	!
		
	COMMON	/FORMS/		Number_of_Forms,	!
     -				Number_of_Items(Isize),	!
     -				Iorder_of_Items(Isize,Isize),!
     -				Cform(Isize),		!
     -				Cform_Name(Isize)	!

	COMMON	/Record_buf/	Record_Length,Record	!
	COMMON	/Screen/	Iline,Itop_line,Ibot_Line
	COMMON	/SORT/		Key_Info(41)		!

	Istart=1					! Starting loc in rec
	Istart0=1					!
	Isort=0						! Assume we don't sort
	Itop_line=3					! Define input window
	Ibot_line=15					!   for common
	Iline=3						! Where to start
	Ierase=4					! Erase to end of line
	CALL SCREEN(Cnull,Iterminal,Itop_Line,1,1)	! Erase window
	IF(Record(1:10).NE.'          ')	THEN	! If record not null
	  CALL SCREEN(Cnull,Iterminal,Ibot_Line+2,1,4)	!   Erase to EOL
	  CALL Display_Record(Iform)			!   Display rec
	END IF						! before any prompting
10	DO I=1,Number_of_Entries			!
	  K=I						!
	  CALL VPW_SCROLL(Iterminal,Itop_line,Ibot_Line,0)! Set scroll wind
15	  CONTINUE					!

	  Imtch1=IAND(Match_type(K),Match_Protected)	! Is it prot. field
	  Imtch2=IAND(Match_type(K),Match_List)		! Is it prot. field
	  J=Field_Name_Length(K)			! Length of Prompt1
	  Jlen2=J					!  " of 2nd prompt
	  If(Imtch2.EQ.Match_List   		.OR.	!
     -	     Imtch1.EQ.Match_Protected)
     -		CALL String_Length(Cmatch_String(K),Jlen2)
	  Jlen=J					! Save length for later
	  IF(Imtch1 .NE. Match_Protected	.AND.
     -       Imtch2 .NE. Match_List)		THEN	! If not, simple prompt
	     Cprompt=Cfield_Name(K)(1:J)//': '		! tack on delimeter
	     J=J+2					!
	  ELSE						!
	     Cprompt=Cfield_Name(K)(1:J)//' ('//Cbold//' '//! If it is tailor
     -               Cmatch_String(K)(1:Jlen2)//Cnorm//' ): '! the prompt
	     CALL STRING_LENGTH(Cprompt,J)		! find total length
	     J=J+1					!
	  END IF					!
	  Imtch=IAND(Match_Type(K),Match_Exit_on_Null)	! Is it Exit_on_Null

20	  CALL SCREEN(Cprompt(1:J),Iterminal,Iline,1,Ierase)! Prompt/erase
	  Ierase=4					! Erase to EOL
	  CALL Read_String(Ctmp,Nchar)			! Get reply
	  IF(Iterminal.GE.96	.AND.			! If VT100
     -	     Error_Flag)			THEN	! and Error flag set
	    Error_Flag=.FALSE.				! reset flag
	    CALL SCREEN(Cnull,Iterminal,24,1,1)		! Erase line
	  END IF					!

C	  CALL CHECK_ESCAPE(Ctmp(1:nchar),Nchar,Iterminal,Helptxt,
C     -			^Z-return,
C     -			Backup_return,
C     -			Done_return,
C     -			Help/VPW_return)
        KK=Field_Help_Length(K)				!
	CALL Check_Escape(Ctmp,Nchar,Iterminal,Cfield_Help(K)(1:KK),!
     -			*9999,				! ^z Return
     -			*50,				! Backup Return
     -			*2000,				! Done_return
     -			*75)				! Help/VPW return

          GOTO 100					! No Escape seq
50	  IF(K.GT.1) K=K-1				! If Backup mode
	  GOTO 15					! and try again
75	  IF(Iterminal.GE.96)			THEN	! If VT100 term
	    CALL SCREEN(Cnull,Iterminal,Ibot_Line+2,1,0)!   Position 
	    CALL Display_Record(Iform)			!   Display rec
	  ELSE 						!
	    TYPE *,' '					! Insert space if Hcpy
	  END IF					!
	  GOTO 20					!


100	  IF(Imtch.EQ.Match_Exit_on_Null	.AND.	! If Exit_on_null
     -       Nchar.EQ.0	.AND.				!  a <CR> input and
     -	     Imode.EQ.Imode_Add)		THEN	!  add mode then
				Istatus=0		!  Show <CR> hit
				RETURN			!  and return
	  END IF					!

	  IF(Nchar.GT.Field_Length(K))		THEN	! If input too long
	    Error_Flag=.TRUE.				!
	    Ctmp='VPW -- '//Cbold//Cfield_name(K)(1:Jlen+2)//Cnorm//
     -	         ' exceeds allowed length'		!
	    CALL String_Length(Ctmp,Ilength)		!
	    CALL SCREEN(Ctmp(1:Ilength),Iterminal,24,1,1)! Output error msg
	    IF(Iterminal.LT.96)	TYPE *,' '		! Skip line if Hcpy
	    GOTO 20					! Prompt again
	  END IF

C
C .. Do match checking if requested
C
C ..                  Check for Required Fields
C
	  Imtch=IAND(Match_type(K),Match_Required)	!
	  IF(Nchar.EQ.0			.AND.		! If <CR> entered
     -	     Imode.EQ.Imode_Add		.AND.		! and if in add mode
     -	     Imtch.EQ.Match_Required)		THEN	! If input required
	    Error_Flag=.TRUE.				!
	    Ctmp='VPW -- '//Cbold//Cfield_name(K)(1:Jlen+2)//Cnorm//
     -	         ' is required input. Type ^Z to exit.'//Cbell
	    CALL String_Length(Ctmp,Ilength)		!
	    CALL SCREEN(Ctmp(1:Ilength),Iterminal,24,1,1)! Output error msg
	    IF(Iterminal.LT.96)	TYPE *,' '		! Skip line if Hcpy
	    GOTO 20					! Prompt again
	  END IF
C
C ..                  Check for Protected Fields
C
	  Imtch=IAND(Match_type(K),Match_Protected)	! Is it prot. field
	  IF(Imtch.EQ.Match_Protected	.AND.		! If protected field
     -	     Nchar.GT.0)			THEN	!  and data entered
	    DO Ichk=1,Nchar				! for each char ent
	      IF(Cmatch_string(K)(Ichk:Ichk).NE.'_' .AND.! If have overwritten
     -	         Cmatch_String(K)(Ichk:Ichk).NE.Ctmp(Ichk:Ichk)) ! a protected
     -						THEN	! field, then warn
	        Error_Flag=.TRUE.			!
	        Ctmp='VPW -- Re-enter data in required format: '//
     -		      Cbold//Cmatch_String(K)(1:Field_Length(K))//
     -		      Cnorm//Cbell
	        CALL String_Length(Ctmp,Ilength)	!
	        CALL SCREEN(Ctmp(1:Ilength),Iterminal,24,1,1)! Output error msg
	        IF(Iterminal.LT.96)	TYPE *,' '	! Skip line if Hcpy
	        GOTO 20					! Prompt again
	      END IF					!
	    END DO					!
	  END IF

C
C ..                  Check for one of list of items
C     The list of items we are matching against are each seperated
C     from each other by at least 1 space, and the last item has a 
C     trailing space.  In this way we can match an item by appending
C     a space to the input string and using the INDEX function.
C
	  Imtch=IAND(Match_type(K),Match_List)		! Is it Match list?
	  IF(Imtch.EQ.Match_List	.AND.		! If protected field
     -	     Nchar.GT.0)		THEN		! and data entered
	      CALL String_Length(Cmatch_String(K),Jlen2)! Get length of list
	      Ipos=INDEX(Cmatch_String(K)(1:Jlen2)//' ',! A match?
     -	                 Ctmp(1:nchar)//' ')
	      IF(Ipos.EQ.0)			THEN	! If not present
	        Error_Flag=.TRUE.			!
	        Ctmp='VPW -- Input data must be one of: '//
     -		      Cbold//Cmatch_String(K)(1:Jlen2)//
     -		      Cnorm//Cbell
	        CALL String_Length(Ctmp,Ilength)	!
	        CALL SCREEN(Ctmp(1:Ilength),Iterminal,24,1,1)! Output error msg
	        IF(Iterminal.LT.96)	TYPE *,' '	! Skip line if Hcpy
	        GOTO 20					! Prompt again
	      END IF					!
	  END IF

C
C ..                  Check for Legal date string
C
	  Imtch=IAND(Match_type(K),Match_Date)		! Is it Match list?
	  IF(Imtch.EQ.Match_Date	.AND.		!If DATE field
     -	     Nchar.GT.0)			THEN	!  and data entered
	      CALL Upper_Case(Ctmp(1:Nchar))		! Make upper case
	      Istatus=SYS$BINTIM(Ctmp(1:nchar),Timbuf)	! Try converting
	      IF(Istatus.NE.1	.OR.			! If not legal date
     -		 Nchar.LE.10)			THEN	!  or too short
	         Error_Flag=.TRUE.			!
	        Ctmp='VPW -- Invalid date format. Enter dates as: '//
     -		      Cbold//'DD-MMM-YYYY'//
     -		      Cnorm//Cbell
	        CALL String_Length(Ctmp,Ilength)	!
	        CALL SCREEN(Ctmp(1:Ilength),Iterminal,24,1,1)! Output error msg
	        IF(Iterminal.LT.96)	TYPE *,' '	! Skip line if Hcpy
	        GOTO 20					! Prompt again
	      END IF					!
	  END IF


C
C ..	Check for range check
C
	  Imtch=IAND(Match_type(K),Match_Range)		! Is it Match list?
	  IF(Imtch.EQ.Match_Range	.AND.		!If range check
     -	     Nchar.GT.0)			THEN	!  and data entered
	     READ(Ctmp(1:nchar),200,ERR=8000)Rval	!
200	     FORMAT(F15.6)
	     CALL String_Length(Cmatch_String(K),Jlen)	! Get length
	     Ipos=INDEX(Cmatch_String(K)(1:Jlen),'/')	! Search for delimiter
	     IF(Ipos.EQ.0)			THEN	!
	        Ctmp='VPW -- Invalid range specifier in prompt library'//
     -		      Cbell				!
	        CALL String_Length(Ctmp,Ilength)	!
	        CALL SCREEN(Ctmp(1:Ilength),Iterminal,24,1,1)! Output error msg
	        IF(Iterminal.LT.96)	TYPE *,' '	! Skip line if Hcpy
	        GOTO 20					! Prompt again
	      END IF					!
	      READ(Cmatch_String(K)(1:Ipos-1),200,ERR=8010)Loval	!
	      READ(Cmatch_String(K)(Ipos+1:Jlen),200,ERR=8020)Hival	!
	      IF(Rval.LT.Loval	.OR.			! If not in range
     -		 Rval.GT.Hival)			THEN	!  
	         Error_Flag=.TRUE.			!
	        Ctmp='VPW -- Input data outside of range of: '//
     -		      Cbold//Cmatch_string(K)(1:Ipos-1)//Cnorm//
     -		      ' to '//
     -		      Cbold//Cmatch_string(K)(Ipos+1:Jlen)//
     -		      Cnorm//Cbell
	        CALL String_Length(Ctmp,Ilength)	!
	        CALL SCREEN(Ctmp(1:Ilength),Iterminal,24,1,1)! Output error msg
	        IF(Iterminal.LT.96)	TYPE *,' '	! Skip line if Hcpy
	        GOTO 20					! Prompt again
	      END IF					!
	  END IF


C
C .. Response must be OK
C
1000	  Istart=Istart0				!
	  DO Idx=1,K-1					! Locate where to put
	    Istart=Istart+Field_Length(Idx)		!  data in record
	  END DO					!
	  IF(NCHAR.GT.0) 			THEN	! Insert data (if any)
	    IF(Key_Info(1).GT.0)		THEN	! If sorting req then
	      DO Ikey=1,Key_Info(1)			!  is this a sort field
	       IF(Istart.EQ.Key_Info(4+4*(Ikey-1))) Isort=1! If yes, must sort
	      END DO					!
	    END IF					!
     	    Record(Istart:Istart+Field_Length(K)-1)=Ctmp(1:Nchar)! load record
	  END IF
	  Istart=Istart+Field_Length(K)			! Point to next field
	  Iline=Iline+1					! and next term line
	  IF(Iline .GT. Ibot_Line)		THEN	! Don't index if would
	      Iline=Iline-1				! put us outside scroll
	  END IF					! window
C ..
C .. Keep record updated on screen after each input
C ..
	  IF(Iterminal.GE.96)			THEN	! If VT100 term
	    IF(Error_Flag .OR. Nchar.GT.0)	THEN	!
	      CALL SCREEN(' ',Iterminal,Ibot_Line+2,1,1)! Erase from top line
	      IF (K.LT.Number_of_Entries)CALL Display_Record(Iform)! Display rec
	    END IF					!
	  END IF					!

	  IF(K.LT.I)				THEN	! If we have backed up
	     K=K+1					!  then finish loop
	     GOTO 15					!  so all entries dsply
	  END IF					!

	END DO

2000	IF(Iterminal.GE.96)			THEN 	! If VT100
	   CALL SCREEN(' ',Iterminal,Itop_line,1,1)	! Erase from top line
	ELSE						!
	   TYPE *, ' '					! Else skip a line
	END IF						!

	Istatus=1+Isort					! We have something
	RETURN

8000	Error_Flag=.TRUE.				!
	Ctmp='VPW -- Error. Non-numeric data entered.'	!
     -		//Cbell
	CALL String_Length(Ctmp,Ilength)		!
	CALL SCREEN(Ctmp(1:Ilength),Iterminal,24,1,1)	! Output error msg
	IF(Iterminal.LT.96)	TYPE *,' '		! Skip line if Hcpy
	GOTO 20						! Prompt again
8010	Ctmp='VPW -- LoVal in Prompt Data is not in valid format: '//
     -	     Cbold//Cmatch_String(K)(1:Ipos-1)//Cnorm//Cbell
	GOTO 8030					!
8020	Ctmp='VPW -- HiVal in Prompt Data is not in valid format: '//
     -	     Cbold//Cmatch_String(K)(Ipos+1:Jlen)//Cnorm//Cbell
	GOTO 8030					!
8030	Error_Flag=.TRUE.				!
	CALL String_Length(Ctmp,Ilength)		!
	CALL SCREEN(Ctmp(1:Ilength),Iterminal,24,1,1)	! Output error msg
	IF(Iterminal.LT.96)	TYPE *,' '		! Skip line if Hcpy
	GOTO 20						! Prompt again
9999	Istatus=-1					! ^Z return
	RETURN
	END

