C+
C	VPW_Load_Prompts
C	
C	This subroutine is designed to load the prompts and the field
C	widths of the data fields from the text library into the 
C	PROMPT common block.  The text library is SYS$SYSDEVICE:[VPW]VPW.TLB.
C	The format of the call is to specify the name of the prompt module
C	desired to use.  The format of the prompt text is
C
C 	<prompt_String>|<Field_Width>|<Help_module_name>|<match_type>|
C        <Match_String>
C	
C	The first two "|"'s are required delimiters.  If the last two fields
C	are omitted, null Match_type and string are defaulted. If a 
C	Match_String is used, its field width is forced to the field width for 
C	the input string, padded with trailing spaces as required.  Up to 20 
C	prompts and individual help texts, may be specified.  This value may 
C	easily be increased by increasing the size of the common block.  The
C	HELP text module name is referenced by VPW_HELP and is stored in the
C	HELP library SYS$VPWFILES:VPW.HLB.  The Field_width is the
C	size of the record (ASCII bytes) into which data will be stored.
C	The order of data storage in the final record is goverened by
C	the order of prompts.  The HELP module name and the prompts can
C	both be up to 50 characters long.  
C
C	Example:	CALL VPW_Load_Prompts('PHONEBOOK_PROMPTS')
C
C	Please note that the common block is required not only in this
C	module, but in all modules referencing either the record definitions,
C	the help, or the prompts.
C
C				Match Types
C
C	Match types are designed to be additive.  Consequently the numbers for
C	Match Types increase in binary order, 0,1,2,4,8...  A given input
C	may have more than one match type attribute.  
C
C	None:		If the Match_type is 0, no match checking is done and 
C			the input is not required.
C	Exit on Null:	If an input is not supplied, the input data routine
C			will exit without entering the data if in add mode.
C			If the data is not required, a carriage return in
C			data entry is treated as a No-op and input data is
C			read for the next available field.
c	Required:	The input data is absolutely required in add mode,
C			but optional in update mode.  If the data is not
C			supplied the routine will reprompt. The only way
C			to exit is ^Z.
C	Protected:	A protection field is a character string. Any location
C			of the string which contains other than a underline 
C			must be filled with only the character in that field.
C			Spaces containing an underline may be filled with any
C			character.
C	One of List:	Entries are matched against a list of strings
C			seperated by spaces.  The entry is accepted only
C			if there is an exact match.
C	Date:		Verify that the entered string is a valid VMS
C			date string.
C	Range check:	The input value is converted to a floating point
C			value and compared with the Lo/High values in the
C			string.  The format for storage is LOW/HIGH.  The
C			"/" seperates the values.  The entry is allowed
C			If and only if the entered value is >= Low value
C			and <= High value.
C
C	Definition of MATCH Types:
C	
C		Match_Type  =	0	Don't do match checking
C			    =	1	Exit subroutine on null
C			    =	2	Parameter must be present
C			    =	4	Do not modify protected fields
C			    =	8	Match one of list
C			    =  16	Verify input is a date string
C			    =  32	Range check input
C
C	Written by:
C			James G. Downward
C			KMS Fusion, Inc.
C			P.O. Box 1567
C			Ann Arbor, Mich. 48106
C			14-Jan-1983
C
C-

	SUBROUTINE VPW_Load_Prompts(Cname)
C	
C		IDENT /2/
C

	IMPLICIT INTEGER*4	(A-Z)
	CHARACTER*(*)		Cname
	CHARACTER*1		Cesc,Cbell,Cnull
	CHARACTER*4		Cbold,Cnorm
	Character*4		Cnumber
	CHARACTER*40		Clibfile
	CHARACTER*132		Crecord
C 
C -------------------------------------------------------
	PARAMETER		(Isize=20)


	CHARACTER*50		Cfield_name
	CHARACTER*50		Cfield_Help
	CHARACTER*50		Cmatch_String
	CHARACTER*50		Cfile				! File name
	CHARACTER*80		Ctitle				! Program title
	CHARACTER*512		record
	INTEGER*4		Field_name_Length
	INTEGER*4		Field_length
	INTEGER*4		Field_Help_Length
	INTEGER*4		Match_Type
	INTEGER*4		Record_Length

	COMMON	/Constants/	Cesc,Cbell,Cnull,Cbold,Cnorm
	COMMON	/Control/	Cfile, Ctitle			!
	COMMON	/PROMPTS/	Number_of_Entries,
     -				Field_Name_Length(Isize),
     -				Field_length(Isize),
     -				Field_Help_Length(Isize),
     -				Match_Type(Isize),
     -				Cfield_name(Isize),
     - 				Cfield_Help(Isize),
     -				CMatch_String(Isize)

	COMMON	/Record_buf/	Record_Length,Record


C ----------------------------------------------------------------
C

	INTEGER*4	IPTR(2)				! For LBR$ routine
	INTEGER*4	Outdes(2)			! Output string descrip
	INCLUDE 'SYS$LIBRARY:LBRDEF.FOR/NOLIST'		! 
	INCLUDE 'SYS$LIBRARY:RMSDEF.FOR/NOLIST'		!

	Clibfile='SYS$VPWFILES:VPW.TLB'			!

	IF(Ichar(Cbell).NE.7)			THEN	! Check to see if
	  TYPE *,'VPW -- Common block /CONSTANTS/ not defined.'! common block
	  GOTO 9000					! is ok
	END IF						!

	CALL STRING_LENGTH(Cname,Iname_Length)		! Get string length
10	Istatus = LBR$INI_CONTROL(Library_Index,	! Initialize library
     -		                  LBR$C_READ,LIB$C_TYP_TXT)! routines to read
	IF(.NOT. Istatus) 			THEN	! a text lib.  If fail
	  TYPE *,'VPW -- LBR$INI_CONTROL failed.'	!  warn user and exit
	  GOTO 9000					!
	END IF						!

	Istatus=LBR$OPEN(Library_Index,Clibfile)	! Open up the library
	IF(.NOT.ISTATUS) 			THEN	! If menu can not be
	  WRITE(*,20)Clibfile				!  output then exit
20	  FORMAT(' VPW -- Fatal, ',A,/,
     -	         '        can not be opened to extract control table.')!
	  GOTO 9000					!
	END IF						!
							!
	Istatus = LBR$LOOKUP_KEY(Library_Index,		! Find the txt module 
     -			         Cname(1:Iname_Length),Iptr)! in the library
	IF(.NOT.Istatus)			THEN	! If fails warn user
	  WRITE(*,30)Cname(1:Iname_Length),Clibfile	! and exit
30	  FORMAT(' VPW -- Fatal, control module, ',A,' can',/,	!
     -           '        not be found in ',A)		!
	  GOTO 9000
	END IF

	Istatus=LBR$GET_RECORD(Library_Index,Ctitle,Outdes)! Read prog title
	IF(.NOT.Istatus)			THEN	! If fails, fatal
	  WRITE(*,40)Clibfile				! and exit
40	  FORMAT(' VPW -- Fatal, program title can',/,	!
     -           '        not be found in ',A)		!
	  GOTO 9000					!
	END IF						!
	Istatus=LBR$GET_RECORD(Library_Index,Cfile,Outdes)! Get data filename
	IF(.NOT.Istatus)			THEN	! If fails, fatal
	  WRITE(*,50)Clibfile				! and exit
50	  FORMAT(' VPW -- Fatal, program data file can',/,!
     -           '        not be found in ',A)		!
	  GOTO 9000					!
	END IF						!
	Istatus=LBR$GET_RECORD(Library_Index,Crecord,Outdes)! Read check flag
	IF(.NOT.Istatus)			THEN	! If fails, fatal
	  WRITE(*,60)Clibfile				! and exit
60	  FORMAT(' VPW -- Fatal, Invalid format for prompt',/,	!
     -           '        module in ',A)		!
	  GOTO 9000					!
	END IF						!
	IF(Crecord(1:1).NE.'/')			THEN	!
	  WRITE(*,60)Clibfile				! warn and exit
	  GOTO 9000
	END IF	  

	Number_of_Entries=0				!
100	Istatus=LBR$GET_RECORD(Library_Index,Crecord,Outdes)	! Read LBR rec
	IF(Istatus.EQ.RMS$_EOF		.OR.		! If EOF or if some
     -     Istatus.NE.1)		       GOTO 200	!  other error, stop

	N=Outdes(1)					! Number of characters
	IF(N.EQ.0)			GOTO 100	! Skip null records
	IF(Crecord(1:1).EQ.'!')		GOTO 100	! Skip Comments
	Number_of_Entries=Number_of_Entries+1		! Count # of records
	I=Number_of_Entries
	Ipos=INDEX(Crecord,'|')				! Find first delimiter
	IF(Ipos.EQ.0)			THEN		! If =0, bad record
	   TYPE *, 'VPW_Load_Prompts -- Record missing first "|".'
	   GOTO 9000					! die
	END IF						!
	Ipos2=INDEX(Crecord(Ipos+1:N),'|')		! Find 2nd delimiter
	IF(Ipos2.EQ.0)			THEN		! If =0, bad record
	   TYPE *,'VPW_Load_Prompts -- Record missing second "|"'
	   GOTO 9000					! also die
	END IF						!
	Ipos3=0						! Both of these
	Ipos4=0						! are optional
	Ipos2=Ipos2+Ipos				! abs pos of ',' of #2
	Ipos3=INDEX(Crecord(Ipos2+1:N),'|')		! Find 3rd delimiter
	IF(Ipos3.GT.0) 				THEN	! If have 3rd "|"
		Ipos3=Ipos3+Ipos2			! Abs pos of 3d "|"
     		Ipos4=INDEX(Crecord(Ipos3+1:N),'|')	!  then find 4th "|"
		IF(Ipos4.GT.0) Ipos4=Ipos4+Ipos3	! Abs pos of 4th "|"
	END IF


	Field_name_Length(I)=Ipos-1			! Calc length 1st field
	Cfield_name(I)=Crecord(1:Field_name_Length(I))	! Load ASCII string
	Cnumber=Crecord(Ipos+1:Ipos2-1)			! Find intended length
	Nchar=Ipos2-Ipos-1				! # of char in number
	READ(Cnumber(1:Nchar),115)Field_Length(I)	! convert to integer
115	FORMAT(I4)					!
	IF(Ipos3.EQ.0)			THEN		! if fields omitted
	     NN=N					! set proper length
	ELSE						!
	     NN=Ipos3-1					! Loc 3d "|"-1
	END IF
	Field_Help_Length(I)=NN-Ipos2			! Length of Help string
	Cfield_Help(I)=Crecord(Ipos2+1:NN)		! Help text to use
C	WRITE(*,120)Field_name_length(I),Cfield_Name(I),Field_length(I),
C     -			Field_Help_Length(I),Cfield_Help(I)
C120	FORMAT(' Field_Name_Length =',I4,
C     -		/,' Cfield =',A,
C     -		/,' Field_Length=',I4,
C     -		/,' Field_Help_Length=',I4,
C     -		/,' Cfield_Help=',A)

	IF(Ipos3.EQ.0)			THEN		!
		Match_Type(I) = 0			!
		Cmatch_String(I) = ' '			! Null it out
	ELSE						!

	   IF(Ipos4.EQ.0)			THEN	! if fields omitted
	     NN=N					! set proper length
	   ELSE						!
	     NN=Ipos4-1					! Loc 4th "|"-1
	   END IF					!

	  Cnumber=Crecord(Ipos3+1:NN)			! Find intended length
	  Nchar=NN-Ipos3				! # of char in number
	  READ(Cnumber(1:Nchar),115)Match_Type(I)	! convert to integer
	  IF(Ipos4.GT.0)			THEN	!
	    IF(Match_Type(I).EQ.2)		THEN	! If "protected field"
	       Ilen=Field_Length(I)			!  make = input length
	    ELSE					! else transfer it all
	       Ilen=LEN(Cmatch_String(I))		!
	    END IF
	    Cmatch_String(I)(1:Ilen)=Crecord(Ipos4+1:N)! Len of match string
	  END IF
	END IF						!
C	IF(ILEN.EQ.0) ILEN=1
C	WRITE(*,125)Match_Type(I),Cmatch_String(I)(1:Ilen)
C125	FORMAT(' Match_type =',I4,
C     -		/,' Match_String =',A,'*')
C	TYPE *, ' '

	GOTO 100

200	Istatus=LBR$CLOSE(Library_Index)		! close when done
	Record_Length=0					! initialize
	DO I=1,Number_of_Entries			! for each entry
	  Record_Length=Record_Length+Field_Length(I)	! Compute the total 
	END DO						!  length of record

1000	RETURN						! All done

9000	TYPE *,' '					!
	TYPE *,'Press '//Cbold//'RETURN'//Cnorm//' to continue'
	READ(*,9010,END=9999)Cans			!
9010	FORMAT(A)					!
9999	CALL EXIT
	END
