C+
C	VPW_Load_Sort
C	
C	This subroutine is designed to load Sort control data from the VPW
C	text library, into the sort control array.  The text library is 
C
C			SYS$VPWFILES:VPW.TLB.  
C
C	The format of the call is to specify the name of the sort module
C	desired to use and the Sort control array.  The format of a sort 
C	control array should be a list of up to 41 numbers.  The first
C	number is a number from 0-10.  0 means no sorting to be done.
C	1-10 represent the number of keys to sort on. For each key to
C	sort on, a block of 4 numbers (seperated with commas) must exist.
C
C	#_of_keys,Data_type,Ascend/Descend,Start_Pos,Length,......
C		  _________________________________________
C
C		   <-- the required 4 words of info/key -->
C
C	The data is returned in an I*2 array of dimension 41 which is passed
C	in the common block SORT.  Information on how to set up the blocks of
C	numbers may be obtained in the VAX SORT manual, Section 3.2.2.
C	
C	Example:	CALL VPW_Load_Sort('PHONEBOOK_SORT')
C
C
C-

	SUBROUTINE VPW_Load_Sort(Cname)

	IMPLICIT INTEGER*4	(A-Z)			! Assume I*4 
	CHARACTER*(*)		Cname			! Read this module
	CHARACTER*1		Cesc,Cbell,Cnull	!
	CHARACTER*4		Cbold,Cnorm		!
	CHARACTER*30		Clibfile
	CHARACTER*255		Crecord			! Input buffer
	INTEGER*2		Key_Info		!
	INTEGER*4	IPTR(2)				! For LBR$ routine
	INTEGER*4	Outdes(2)			! Output string descrip

	COMMON /SORT/	Key_Info(41)			! For Sort key info
	COMMON	/Constants/	Cesc,Cbell,Cnull,Cbold,Cnorm

	INCLUDE 'SYS$LIBRARY:LBRDEF.FOR/NOLIST'		! 
	INCLUDE 'SYS$LIBRARY:RMSDEF.FOR/NOLIST'		!

	IF(Ichar(Cbell).NE.7)			THEN	! Is common blk loaded
	  TYPE *,'VPW -- Common block /Constants/ not loaded.'
	  GOTO 9000					!
	END IF
	Clibfile='SYS$VPWFILES:VPW.TLB'			! Library file name
	CALL STRING_LENGTH(Cname,Iname_Length)		! Get string length
10	Istatus = LBR$INI_CONTROL(Library_Index,	! Initialize library
     -		                  LBR$C_READ,LIB$C_TYP_TXT)! routines to read
	IF(.NOT. Istatus) 			THEN	! a text lib.  If fail
	  TYPE *,'VPW -- LBR$INI_CONTROL failed.'	!  warn user and exit
	  GOTO 9000					!  Exit block
	END IF						!

	Istatus=LBR$OPEN(Library_Index,Clibfile)	! Open up the library
	IF(.NOT.ISTATUS) 			THEN	! If menu can not be
	  WRITE(*,20)Clibfile				!  output then exit
20	  FORMAT(' VPW -- Fatal, ',A,/,			!
     -	         '        can not be opened to extract control table.')!
	  GOTO 9000					! Exit block
	END IF						!
							!
	Istatus = LBR$LOOKUP_KEY(Library_Index,		! Find the txt module 
     -			         Cname(1:Iname_Length),Iptr)! in the library
	IF(.NOT.Istatus)			THEN	! If fails warn user
	  WRITE(*,30)Cbold,Cname(1:Iname_Length),Cnorm,Clibfile	! and exit
30	  FORMAT(' VPW -- Fatal, Sort module, ',A,A,A,' can',/,	!
     -           '        not be found in ',A)		!
	  GOTO 9000					! Exit block
	END IF


100	Istatus=LBR$GET_RECORD(Library_Index,Crecord,Outdes)	! Read LBR rec
	IF(Istatus.EQ.RMS$_EOF		.OR.		! If EOF or if some
     -     Istatus.NE.1)		       THEN	!
	   Key_Info(1)=0				! Don't sort in null
	   RETURN					! module
	END IF

	N=Outdes(1)					! Number of characters
	IF(N.EQ.0)			GOTO 100	! Skip null records
	IF(Crecord(1:1).EQ.'!')		GOTO 100	! Skip Comments
	Ipos=INDEX(Crecord,',')				! Find first comma
	IF(Ipos.EQ.0)			THEN		!
	   Key_Info(1)=0				! Don't sort
	   Return					!
	END IF
	
	READ(Crecord(1:Ipos-1),110)Key_Info(1)		!
110	FORMAT(I2)					!
	Ilast=4*Key_Info(1)+1				! Last entry
	READ(Crecord(Ipos+1:N),120) (Key_Info(J),J=2,Ilast)!
120	FORMAT(<Ilast-1>I4)

200	Istatus=LBR$CLOSE(Library_Index)		! close when done
1000	RETURN						! All done

9000	TYPE *, ' '					!
	TYPE *, 'Press '//Cbold//'RETURN'//Cnorm//' to continue'!
	READ(*,9010,END=9999)Cans			!
9010	FORMAT(A)					!
9999	CALL EXIT					!
	END
