C+
C	VPW_MENU
C	
C	The routine VPW_MENU is designed to display menu text (or any
C	text) from SYS$VPWFILES:VPW.TLB onto SYS$OUTPUT.  This is to ease 
C	the coding of menus and text displays.  The calling format is
C	
C	CALL VPW_MENU(Ctopic,Choice,Nchar,Iterm,Iline,Icol,Ictrl)
C
C	Where:
C		Ctopic  -  A character string.  Ctopic may either be a
C			   character literal or a dimensioned character
C			   variable.  VPW_MENU calls STRING_LENGTH to
C			   find the length of the string (truncated to
C			   the first trailing space).
C
C		Choice  -  The returned value of the choice as selected
C			   from the legal list of choices.  A character string.
C
C		Nchar   -  The number of characters in the choice.
C
C		Iterm	-  Terminal type
C
C		Iline   -  Line to start menu on.
C
C		Icol 	-  Column to start menu on.
C
C		Ictl    -  Screen control.  See SCREEN.BAS.
C
C	Any failure in VPW_MENU will force a program exit with an appropriate
C	error message.
C
C	The menu format is as follows:
C
C		Text...text...text...text...text...text
C		Text...text...text...text...text...text
C		Text...text...text...text...text...text
C		Text...text...text...text...text...text
C		/
C		M1 M2 M3 M4 .....
C
C	The "/" must be in column 1 and signifies the end of the text
C	to display.  Immediately following it is a list of key phrases
C	or letters which are allowed input. Each must be followed by
C	exactly 1 space.  The rest of the text (if any is ignored)
C-

	SUBROUTINE VPW_MENU(Ctopic,Choice,Nchar,Iterminal,
     -			    Iline,Icol,Ictrl)
	IMPLICIT INTEGER*4	(A-Z)
	CHARACTER*1	Cnull,Cesc,Cans			! A null to test for
	CHARACTER*4	Cbold,Cnorm,Ctmp		!
	CHARACTER*(*)	Ctopic				! 
	CHARACTER*80	Ctest				! Allowed choice string
	Character*(*)	Choice				! Returned choice
	CHARACTER*80	Cbuffer				! Output buffer
	INTEGER*4	IPTR(2)				! For LBR$ routine
	INTEGER*4	Itopic_Length			!
	INTEGER*4	Outdes(2)			! Output string descrip
	COMMON	/Screen/  Icur_line,Itop,Ibot
	DATA	Cnull	/0/				!
	DATA	Cesc	/27/				!
	INCLUDE 'SYS$LIBRARY:LBRDEF.FOR/NOLIST'		! 
	INCLUDE 'SYS$LIBRARY:RMSDEF.FOR/NOLIST'		!
	Itop=1						!
	Ibot=24						!
	IF(Iterminal.GE.96)			THEN	!
	  Cbold=Cesc//'[1m'				!
	  Cnorm=Cesc//'[0m'				!
	ELSE						!
	  Cbold=Cnull//Cnull//Cnull//Cnull		!
	  Cnorm=Cbold					!
	END IF						!
	CALL STRING_LENGTH(Ctopic,Itopic_Length)	! Get string length
10	Istatus = LBR$INI_CONTROL(Library_Index,	! Initialize library
     -		                  LBR$C_READ,LIB$C_TYP_TXT)! routines to read
	IF(.NOT. Istatus) 			THEN	! a text lib.  If fail
	  TYPE *,'VPW -- LBR$INI_CONTROL failed.'	!  warn user and exit
	  ACCEPT *,Cbuffer				!
	  CALL EXIT					!
	END IF						!

	Istatus=LBR$OPEN(Library_Index,'SYS$VPWFILES:VPW.TLB')! Open library
	IF(.NOT.ISTATUS) 			THEN	! If menu can not be
	  WRITE(*,20)					!  output then exit
20	  FORMAT(' VPW -- Fatal, SYS$VPWFILES:VPW.TLB could not be opened',
     -	         ' to display menu.')			!
	  GOTO 9500					!
	END IF						!
							!
	Istatus = LBR$LOOKUP_KEY(Library_Index,		! Find the txt module 
     -			         Ctopic(1:Itopic_Length),Iptr)! in the library
	IF(.NOT.Istatus)			THEN	! If fails warn user
	  WRITE(*,30)Ctopic(1:Itopic_Length)		! and exit
30	  FORMAT(' VPW -- Fatal, text module, ',A,	!
     -          ' can not be found in SYS$VPWFILES:VPW.TLB')!
	  GOTO 9500					! EXIT
	END IF

	CALL SCREEN(' ',Iterminal,Iline,Icol,Ictrl)	!
	Icntr=Iline					!
100	Istatus=LBR$GET_RECORD(Library_Index,Cbuffer,Outdes)	! Read LBR rec
	IF(Istatus.EQ.RMS$_EOF		.OR.		! If EOF or if some
     -     Istatus.NE.1)		       GOTO 9000!  other error, stop
	IF(Icntr.GT.22)				THEN	!
	  CALL PAGE					!
	  TYPE *,'VPW -- Fatal.  The menu is too long.' !
	  GOTO 9500					! EXIT
	END IF						!
	IF(Cbuffer(1:1).EQ.'/')			GOTO 200! Stop display on /
C
C ...  If dumb terminal, Remove bold and unbold attributes
C
	IF(Iterminal.LT.96)			THEN	! If not VT100
	  Ctmp=Cesc//'[1m'				!
	  I=INDEX(Cbuffer,Ctmp)				! Any bolding
	  IF(I.GT.0) Cbuffer(I:I+3)=Cnull//Cnull//Cnull//Cnull
	  Ctmp=Cesc//'[0m'				!
	  I=INDEX(Cbuffer,Ctmp)				! Any unbolding
	  IF(I.GT.0) Cbuffer(I:I+3)=Cnull//Cnull//Cnull//Cnull
	END IF						!
	CALL Screen(Cbuffer(1:Outdes(1)),Iterminal,Icntr,1,0)!
	IF(Iterminal.LT.96)	WRITE(*,150)
150	FORMAT(' ')
	Icntr=Icntr+1					! Keep track of line
	GOTO 100					! Loop till done
200	Istatus=LBR$GET_RECORD(Library_Index,Cbuffer,Outdes)	! Read LBR rec
	IF(Istatus.EQ.RMS$_EOF		.OR.		! If EOF or if some
     -     Istatus.NE.1)		       GOTO 9000!  other error, stop
	Ctest=Cbuffer(1:Outdes(1))//' '			! Transfer to test fuf
	Istatus=LBR$CLOSE(Library_Index)		! close when done
300	CALL Screen('        Choice: ',Iterminal,Icntr,1,4)! Erase to EOL
	CALL READ_STRING(Choice,Nchar)			!

C
C
C	CALL Check_Escape(Ctmp,Nchar,Iterminal,Chelp,
C     -			*nnn,			! ^z/Alt -
C     -			*mmm,			! Backup(up arror) Return
C     -			*ooo,			! Done (7)_return
C     -			*ppp)			! Help/VPW return PF2/PF4
C
C	      Ctmp	CHARACTER*(*)	Input character string to check
C	      Nchar	INTEGER*4	Number of characters
C					>=  0 not escape seq
C					=  -1 ^Z input
C					=  -2 Escape sequence
C	      Iterminal	INTEGER*4	Terminal type
C	      Chelp	CHARACTER*(*)	Help text to display
C	      *nnn      ......		List of alternate return addresses
C


	IF(Nchar.EQ.-2)				THEN	! If ESC seq
	  Icur_Line=Iline-1				!
	  CALL Check_Escape(Choice,Nchar,Iterminal,Ctopic(1:Itopic_Length),
     -			*9999,			! ^z/Alt -
     -			*10,			! Backup(up arrow) Return
     -			*9999,			! Done (7)_return
     -			*325)			! Help/VPW return PF2/PF4

	  Choice=' '					!
	  Nchar=1					!
	  GOTO 10					!
325	  CONTINUE
	  TYPE *,' '					!
	  CALL Screen(' ',Iterminal,Iline,1,1)		!    erase help info
	  GOTO 10					!    Display menu again
	ELSE IF(Nchar.EQ.-1)			THEN	!
	  CALL EXIT					! Exit on ^Z
	ELSE IF(Nchar.EQ.0)			THEN	! <CR> not allowed
	  GOTO 300					!
	ELSE						!
	  CALL UPPER_CASE(Choice)			! Make all uppercase
	  Istart=1					! Start with first char
	  Itest_Length=Outdes(1)+1			!
400	  Ipos=INDEX(Ctest(Istart:Itest_Length),' ')	! Find space
	  IF(Ipos.EQ.0)			      GOTO 500	! No match found
	  Ipos=Istart+Ipos-1				! real loc of space
	  Imatch_Length=Ipos-Istart			! Length of match 
	  IF(Choice(1:Imatch_Length)	.EQ.		!
     -	     Ctest(Istart:Ipos-1)) 		THEN	! A match
	     Choice=Ctest(Istart:Ipos-1)		! return keword
	     Nchar=Imatch_Length			! and length
	     RETURN					! 
	  END IF					!
	  IF(Ipos.GE.Itest_Length)	      GOTO 500	! String end no match
	  Istart=Ipos+1					! Point at nxt char
	  GOTO 400					!
500	  Cbuffer='VPW -- '//Cbold//Choice(1:Nchar)//Cnorm//!
     -		' is not a valid choice.'		!
	  CALL String_Length(Cbuffer,I)			!
	  CALL SCREEN(Cbuffer(1:I),Iterminal,24,1,0)	!
	  GOTO 300

	END IF						!

1000	RETURN						! All done
9000	CALL PAGE					!
	TYPE *,'VPW -- Fatal.  Premature EOF in menu.'
	GOTO 9500					!
9500	TYPE *, ' '					!
	TYPE *,'Press '//Cbold//'RETURN'//Cnorm//' to continue'
	READ(*,9510,END=9999)Cans			!
9510	FORMAT(A)					!
9999	CALL EXIT					!
	END
